/**
 * @class MoonplayerAutoplay
 *
 * @param {Object} mp                   Object of Moonplayer Class
 * @property {Object} el                DOMElement .media-control-button.media-control-icon
 * @property {Boolean} keepEnabled      Autoplay status
 * @method _triggerButton               Callback function on click on autoplay button
 * @method _getButtonHTML               Returns content of autoplay button
 * @method flush                        Initialize or update autoplay action
 */
function MoonplayerAutoplay(mp) {
    
    var self = this;
    
    this.el = null;
    this.keepEnabled = true;
    
    /**
     * @method _triggerButton
     */
    this._triggerButton = function() {
        self.keepEnabled = !self.keepEnabled;
        
        this.el.className = this.el.className.replace(/\s?autoplay/, '');
        
        if (self.keepEnabled) this.el.className += ' autoplay';
    };
    
    /**
     * @method _getButtonHTML
     */
    this._getButtonHTML = function() {
        return '<svg class="on" xmlns="http://www.w3.org/2000/svg" height="14.028" viewBox="0 0 14.027999 14.027999" width="14.028"><g transform="matrix(.031865 0 0 .031865 0 .013768)"><path d="m434.82 207.28-202.71-202.71c-3.609-3.617-6.655-4.856-9.133-3.713-2.475 1.143-3.712 4.189-3.712 9.137v202.71c-0.949-2.091-2.187-3.901-3.711-5.424l-202.7-202.71c-3.621-3.616-6.664-4.855-9.139-3.712-2.474 1.143-3.711 4.189-3.711 9.137v420.26c0 4.948 1.237 7.994 3.711 9.138 2.474 1.14 5.518-0.1 9.135-3.721l202.7-202.71c1.521-1.711 2.762-3.524 3.711-5.428v202.71c0 4.948 1.237 7.991 3.712 9.131 2.478 1.143 5.523-0.093 9.133-3.714l202.72-202.7c3.61-3.617 5.428-7.901 5.428-12.847-0.002-4.941-1.817-9.223-5.427-12.84z"/></g></svg>' +
            '<svg class="off" xmlns="http://www.w3.org/2000/svg" height="16.303" viewBox="0 0 16.302999 16.302999" width="16.303"><g transform="matrix(.031865 0 0 .031865 .051742 -.0054948)"><path d="m506.2 41.968c-3.617-3.617-7.902-5.426-12.851-5.426h-36.542c-4.948 0-9.232 1.809-12.847 5.426-3.62 3.619-5.427 7.902-5.427 12.851v193.57c-0.955-2.091-2.19-3.899-3.717-5.424l-202.71-202.71c-3.616-3.615-6.658-4.853-9.136-3.709-2.474 1.141-3.711 4.187-3.711 9.135v202.71c-0.95-2.091-2.187-3.899-3.709-5.424l-202.7-202.72c-3.62-3.618-6.664-4.856-9.138-3.712-2.475 1.141-3.712 4.187-3.712 9.135v420.26c0 4.948 1.241 7.998 3.715 9.141 2.474 1.14 5.518-0.099 9.135-3.72l202.71-202.7c1.52-1.708 2.76-3.519 3.709-5.421v202.71c0 4.948 1.237 7.994 3.711 9.134s5.52-0.1 9.136-3.717l202.7-202.72c1.523-1.711 2.762-3.524 3.714-5.428v193.57c0 4.948 1.81 9.229 5.427 12.847 3.614 3.617 7.898 5.425 12.847 5.425h36.546c4.944 0 9.232-1.808 12.85-5.425 3.614-3.617 5.425-7.898 5.425-12.847v-401.98c-0.005-4.956-1.811-9.235-5.428-12.851z"/></g></svg>';
    };
    
    /**
     * @method flush
     */
    this.flush = function() {
    
        var elementParent = mp.documentPlayer.querySelector('.media-control-left-panel');
        
        var element = elementParent.querySelector('.media-control-button.media-control-icon[data-autoplay]');
        
        if (element) element.parentNode.removeChild(element);
        
        this.el = document.createElement('button');
        this.el.setAttribute('data-autoplay', '');
        this.el.innerHTML = this._getButtonHTML();
        this.el.className = 'media-control-button media-control-icon' + (this.keepEnabled ? ' autoplay' : '' );
        
        if (mp.data.isSerial()) {
            elementParent.appendChild(this.el);
            
            this.el.addEventListener('click', function() {
                self._triggerButton();
                mp.player.showMessage( self.keepEnabled ? 'АВТОПЛЕЙ СЕРИЙ' : 'ОДИНОЧНЫЙ РЕЖИМ' );
            });
            this.el.addEventListener('mouseenter', function() {
                mp.player.showMessage( self.keepEnabled ? 'АВТОПЛЕЙ СЕРИЙ' : 'ОДИНОЧНЫЙ РЕЖИМ' );
            });
            this.el.addEventListener('mouseleave', mp.player.hideMessage);
        }

    };
    
    /**
     * @method next
     */
    this.next = function() {
        if (!self.keepEnabled) return false;
        
        var names = mp.playlist.getNextNames();
        
        if (names.season === null) return false;
        
        var link = mp.playlist.getUrlByNames(names.season, names.episode);
        
        mp.load(link, true);
    };
    
}


/**
 * @class MoonplayerBanner
 *
 * @param {Object} mp           Object of Moonplayer Class
 * @method _normalize           Validate and normalize defined options
 * @method _centralizeStyle     To set a centralized styles to element
 * @method _genBannerElements   Generate banner elements by defined options
 * @method _setAction           To define banner actions 
 * @method show                 Create and show the banner
 */
function MoonplayerBanner(mp) {

    /**
     * To validate and normalize list of values:
     * script {String}      String of javascript to execute on head element (default: null)
     * content {String}     Banner content (default: null) also you can pass element id to use the inner content
     * closeText {String}   Value of close button (default: закрыть)
     * timerText {String}   Value of close button if timer is running (pattern %count% will be replaced at seconds) (default: осталось %count% сек.)
     * timer {Number}       Seconds of timer delay (default: 0)
     * assistText {String}  Auxiliary message on banner (default: null)
     * width {Number}       The width property in px (default: 200)
     * height {Number}      The height property in px (default: 200)
     * filled {Boolean}     Hide background when banner is showing (default: true)
     * callback {Function}  This function will be called after the banner will be closed (default: null)
     *
     * @method _normalize
     * @param {Object} opt  Defined banner options
     */
    this._normalize = function(opt) {
        
        if (!opt) return false;
        
        opt.script = typeof opt.script === 'string' && opt.script.length ? opt.script : null;
        opt.content = typeof opt.content === 'string' && opt.content.length ? opt.content : null;
        
        if (opt.content && opt.content.indexOf('#') === 0) {
            var elementContent = document.getElementById(opt.content.substr(1));
            opt.content = elementContent ? elementContent.innerHTML : null;
        }
        
        if (!opt.script && !opt.content) return false;
        
        opt.closeText = opt.closeText || 'закрыть';
        opt.timerText = opt.timerText || 'осталось %count% сек.';
        opt.assistText = opt.assistText || null;
        
        opt.timer = typeof opt.timer === 'number' && opt.timer ? opt.timer : 0;
        
        opt.width = parseInt(opt.width || 200);
        opt.height = parseInt(opt.height || 200);
        
        opt.filled = !!opt.filled;
        
        opt.callback = typeof opt.callback === 'function' ? opt.callback : null;
        
        return opt;
        
    };
    
    /**
     * @method _centralizeStyle
     *
     * @param {Object} element Element to set centralized styles
     * @param {Number} width   The width of element in px (require for calculation)
     * @param {Number} height  The height of element in px (require for calculation)
     */
    this._centralizeStyle = function(element, width, height) {
        
        if (!element) return false;
        
        if (width || height) element.style.position = 'absolute';
        
        if (width) {
            element.style.top = '50%';
            element.style['margin-left'] = -width/2 +'px';
        }
        
        if (height) {
            element.style.left = '50%';
            element.style['margin-top'] = -height/2 +'px';
        }
        
    };
    
    /**
     * @method _genBannerElements
     *
     * @param {Object} options Defined banner options
     * @returns {Object} Associated array with elements
     */
    this._genBannerElements = function(options) {
        
        // script element
        var elementScript = null;
        
        if (options.script) {
            elementScript = document.createElement('script');
            elementScript.innerHTML = options.script;
        }
        
        // content element
        var elementContent = null;
        
        if (options.content) {
            elementContent = document.createElement('div');
            elementContent.className = 'banner-content';
            elementContent.innerHTML = options.content;
        }
        
        // assist element
        var elementAssist = null;
        
        if (options.assistText) {
            elementAssist = document.createElement('div');
            elementAssist.className = 'banner-assist';
            elementAssist.innerHTML = options.assistText;
        }
        
        // root element
        var elementBanner = document.createElement('div');
        elementBanner.className = 'banner';
        elementBanner.className += options.filled ? ' banner-filled' : '';
        
        // close element
        var elementClose = document.createElement('div');
        elementClose.className = 'banner-close';
        
        // the timer element is a child of close element
        var elementTimer = null;
        
        if (options.timer) {
            elementClose.innerHTML = options.timerText.replace(/%count%/, '<span class="banner-timer">'+ options.timer +'</span>');
            elementTimer = elementClose.querySelector('.banner-timer') || null;
        } else
            elementClose.innerHTML = options.closeText;
        
        // style
        var elementToCentralize = options.filled ? elementContent : elementBanner;
        this._centralizeStyle(elementToCentralize, options.width, options.height);
        
        if (elementContent) {
            elementContent.style.width = options.width ? options.width +'px' : '';
            elementContent.style.height = options.height ? options.height +'px' : '';
        }
        
        // join
        if (elementContent) elementBanner.appendChild(elementContent);
        elementBanner.appendChild(elementClose);
        if (elementAssist) elementBanner.appendChild(elementAssist);
        
        return {
            'banner' : elementBanner,
            'content' : elementContent,
            'close' : elementClose,
            'assist' : elementAssist,
            'timer' : elementTimer,
            'script' : elementScript,
        };
        
    };
    
    /**
     * @method _setAction
     *
     * @param {Object} Associated array with elements (required keys: banner, timer, close, script)
     * @param {Object} Defined banner options
     */
    this._setAction = function(elements, options) {
        
        var counter = 0;
        
        function timer() {
            if (--counter < 1) {
                clearInterval(interval);
                timerEnded();
            } else {
                elements.timer.innerHTML = counter;
            }
        }
        
        function timerEnded() {
            elements.close.innerHTML = options.closeText;
            elements.close.className = 'banner-close enabled';
            elements.close.addEventListener('click', removeBanner);
        }
        
        function removeBanner() {
            if (elements.banner) elements.banner.parentNode.removeChild(elements.banner);
            if (elements.script) elements.script.parentNode.removeChild(elements.script);
            if (options.callback) options.callback();
        }
        
        if (options.timer) {
            counter = options.timer;
            var interval = setInterval(timer, 1000);
        } else {
            timer();
        }
        
    };
    
    /**
     * @method show
     *
     * @param {Object} options Defined banner options
     */
    this.show = function(options) {
        
        options = this._normalize(options);
        
        if (!options) {
            mp.warning('[MoonplayerBanner] Normalize was be failed');
            return false;
        }
        
        var elements = this._genBannerElements(options);
        
        if (elements.banner) mp.documentLayer.appendChild(elements.banner);
        
        if (elements.script) document.querySelector('head').appendChild(elements.script);
        
        this._setAction(elements, options);
        
    };
    
}


/**
 * @class MoonplayerData
 *
 * @param {Object} mp Object of Moonplayer Class
 * @property {Object} json Data storage
 * @method load Query data from server
 * @method urlToData Convert iframe url string to data string for send to the server
 * @method isSerial Return true if content type is serial or false
 */
function MoonplayerData(mp) {

	var self = this;

	this.json = null;

	/**
	 * @method load
	 *
	 * @param {String|Object} dataPost Data to be sent to the server
	 * @param {Function} callback Callback function after query
	 */
	this.load = function(dataPost, callback) {
		$.ajax({
			type: 'POST',
			url: mp.options.parserUrl,
			data: dataPost,
			dataType: 'json',
			error: function() {
				mp.error('[MoonplayerData] ajax error');
			},
			success: function(data) {
				if (data.error) mp.error('[MoonplayerData] server error');
				self.json = data;
				document.cookie = data.cookies + ' path=/';
				if (callback) callback(data);
			}
		});
	};

	/**
	 * @method urlToData
	 *
	 * @param {String} url Absolute or relative iFrame url
	 * @returns {String} Data for send to the server
	 */
	this.urlToData = function(url) {
		return url.replace(/(.*)\/(.*?)\/(.*?)\/iframe\??(.*)/g, 'type=$2&token=$3&$4');
	};
    
    /**
     * @method isSerial
     *
     * @returns {Boolean} Return true if content type is serial or false
     */
    this.isSerial = function() {
        return mp.data.json.playlist.seasons.length && mp.data.json.playlist.episodes.length;
    };

}



/**
 * @class MoonplayerEvents
 *
 * @param {Object} mp                               Object of Moonplayer Class
 * @property {Object} list                          The object with a events array is sorted by keys
 * @property {Boolean} keepPreventingEventsDiabled  To disable all PREVENT's events
 *
 * @method _bindOnPlayIntent                        To bind ON_PLAY_INTENT event
 * @method _unbindOnPlayIntent                      To stop listening ON_PLAY_INTENT event
 * @method _triggerOnPlayIntent                     Triggering ON_PLAY_INTENT event
 *
 * @method _bindOnLoadMetadata                      To bind ON_LOAD_METADATA event
 * @method _unbindOnLoadMetadata                    To stop listening ON_LOAD_METADATA event
 * @method _triggerOnLoadMetadata                   Triggering ON_LOAD_METADATA event
 *
 * @method _bindOnFirstClickToPlay                  To bind ON_FIRST_CLICK_TO_PLAY event
 * @method _unbindOnFirstClickToPlay                To stop listening ON_FIRST_CLICK_TO_PLAY event
 * @method _triggerOnFirstClickToPlay               Triggering ON_FIRST_CLICK_TO_PLAY event
 *
 * @method _bindOnFirstPlay                         To bind ON_FIRST_PLAY event
 * @method _unbindOnFirstPlay                       To stop listening ON_FIRST_PLAY event
 * @method _triggerOnFirstPlay                      Triggering ON_FIRST_PLAY event
 *
 * @method _bindOnFirstPause                        To bind ON_FIRST_PAUSE event
 * @method _unbindOnFirstPause                      To stop listening ON_FIRST_PAUSE event
 * @method _triggerOnFirstPause                     Triggering ON_FIRST_PAUSE event
 *
 * @method _bindOnPlay                              To bind ON_PLAY event
 * @method _unbindOnPlay                            To stop listening ON_PLAY event
 * @method _triggerOnPlay                           Triggering ON_PLAY event
 *
 * @method _bindOnPause                             To bind ON_PAUSE event
 * @method _unbindOnPause                           To stop listening ON_PAUSE event
 * @method _triggerOnPause                          Triggering ON_PAUSE event
 *
 * @method _bindOnEnded                             To bind ON_ENDED event
 * @method _unbindOnEnded                           To stop listening ON_ENDED event
 * @method _triggerOnEnded                          Triggering ON_ENDED event
 *
 * @method _bindOnMcRendered                        To bind ON_MC_RENDERED event
 * @method _unbindOnMcRendered                      To stop listening ON_MC_RENDERED event
 * @method _triggerOnMcRendered                     Triggering ON_MC_RENDERED event
 *
 * @method _bindOnMcHide                            To bind ON_MC_HIDE event
 * @method _unbindOnMcHide                          To stop listening ON_MC_HIDE event
 * @method _triggerOnMcHide                         Triggering ON_MC_HIDE event
 *
 * @method _bindOnMcShow                            To bind ON_MC_SHOW event
 * @method _unbindOnMcShow                          To stop listening ON_MC_SHOW event
 * @method _triggerOnMcShow                         Triggering ON_MC_SHOW event
 *
 * @method _triggerOnCreated                        Triggering ON_CREATED event (fire on a this.bind method)
 * @method _triggerOnRefresh                        Triggering ON_REFRESH event (fire on a this.refresh method)
 * @method _triggerPreventPlay                      Call each PREVENT_PLAY's functions (stopPropagation)
 *
 * @method bind                                     To bind each events after player was created
 * @method refresh                                  To refresh events after load new source
 */
function MoonplayerEvents(mp) {

	var self = this;

	this.list = {};
    this.keepPreventingEventsDiabled = false;
    
     /**
	 * @method _bindOnPlayIntent
	 */
	this._bindOnPlayIntent = function() {
		mp.player.engine.core.getCurrentPlayback().once(Clappr.Events.PLAYBACK_PLAY_INTENT, self._triggerOnPlayIntent);
	};
    
    /**
     * @method _unbindOnPlayIntent
     */
    this._unbindOnPlayIntent = function() {
        mp.player.engine.core.getCurrentPlayback().off(Clappr.Events.PLAYBACK_PLAY_INTENT, self._triggerOnPlayIntent);
    };
    
    /**
     * @method _triggerOnLoadMetadata
     */
    this._triggerOnPlayIntent = function() {
        for (var i = 0; i < self.list.ON_PLAY_INTENT.length; i++) {
			if (self.list.ON_PLAY_INTENT[i]) self.list.ON_PLAY_INTENT[i](mp);
		}
    };

	/**
	 * @method _bindOnLoadMetadata
	 */
	this._bindOnLoadMetadata = function() {
		mp.player.engine.core.getCurrentContainer().once(Clappr.Events.CONTAINER_LOADEDMETADATA, self._triggerOnLoadMetadata);
	};
    
    /**
     * @method _unbindOnLoadMetadata
     */
    this._unbindOnLoadMetadata = function() {
        mp.player.engine.core.getCurrentContainer().off(Clappr.Events.CONTAINER_LOADEDMETADATA, self._triggerOnLoadMetadata);
    };
    
    /**
     * @method _triggerOnLoadMetadata
     */
    this._triggerOnLoadMetadata = function() {
        for (var i = 0; i < self.list.ON_LOAD_METADATA.length; i++) {
			if (self.list.ON_LOAD_METADATA[i]) self.list.ON_LOAD_METADATA[i](mp);
		}
        // drop PREVENT_PLAY and ON_FIRST_CLICK_TO_PLAY events (fix autoplay)
        self._unbindOnFirstClickToPlay();
    };

	/**
	 * @method _bindOnFirstClickToPlay
	 */
	this._bindOnFirstClickToPlay = function() {
		mp.player.engine.core.el.addEventListener('click', self._triggerOnFirstClickToPlay, true);
	};
    
    /**
	 * @method _unbindOnFirstClickToPlay
	 */
	this._unbindOnFirstClickToPlay = function() {
		mp.player.engine.core.el.removeEventListener('click', self._triggerOnFirstClickToPlay, true);
	};
    
    /**
	 * @method _triggerOnFirstClickToPlay
	 */
	this._triggerOnFirstClickToPlay = function(e) {
		for (var i = 0; i < self.list.ON_FIRST_CLICK_TO_PLAY.length; i++) {
			if (self.list.ON_FIRST_CLICK_TO_PLAY[i]) self.list.ON_FIRST_CLICK_TO_PLAY[i](mp);
		}
        // Call PREVENT_PLAY's
        self._triggerPreventPlay(e);
        // remove listener
		mp.player.engine.core.el.removeEventListener('click', self._triggerOnFirstClickToPlay, true);
	};

	/**
	 * @method _bindOnFirstPlay
	 */
	this._bindOnFirstPlay = function() {
		mp.player.engine.once(Clappr.Events.PLAYER_PLAY, self._triggerOnFirstPlay);
	};
    
    /**
	 * @method _unbindOnFirstPlay
	 */
	this._unbindOnFirstPlay = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PLAY, self._triggerOnFirstPlay);
	};
    
    /**
     * @method _triggerOnFirstPlay
     */
    this._triggerOnFirstPlay = function() {
        for (var i = 0; i < self.list.ON_FIRST_PLAY.length; i++) {
			if (self.list.ON_FIRST_PLAY[i]) self.list.ON_FIRST_PLAY[i](mp);
		}
    };
    
    /**
	 * @method _bindOnFirstPause
	 */
	this._bindOnFirstPause = function() {
		mp.player.engine.once(Clappr.Events.PLAYER_PAUSE, self._triggerOnFirstPause);
	};
    
    /**
	 * @method _unbindOnFirstPause
	 */
	this._unbindOnFirstPause = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PAUSE, self._triggerOnFirstPause);
	};
    
    /**
     * @method _triggerOnFirstPause
     */
    this._triggerOnFirstPause = function() {
        for (var i = 0; i < self.list.ON_FIRST_PAUSE.length; i++) {
			if (self.list.ON_FIRST_PAUSE[i]) self.list.ON_FIRST_PAUSE[i](mp);
		}
    };
    
    /**
	 * @method _bindOnPlay
	 */
    this._bindOnPlay = function() {
        mp.player.engine.on(Clappr.Events.PLAYER_PLAY, self._triggerOnPlay);
    };
    
    /**
	 * @method _unbindOnPlay
	 */
    this._unbindOnPlay = function() {
        mp.player.engine.off(Clappr.Events.PLAYER_PLAY, self._triggerOnPlay);
    };
    
    /**
     * @method _triggerOnPlay
     */
    this._triggerOnPlay = function() {
        for (var i = 0; i < self.list.ON_PLAY.length; i++) {
			if (self.list.ON_PLAY[i]) self.list.ON_PLAY[i](mp);
		}
    };

	/**
	 * @method _bindOnPause
	 */
	this._bindOnPause = function() {
		mp.player.engine.on(Clappr.Events.PLAYER_PAUSE, self._triggerOnPause);
	};
    
    /**
	 * @method _unbindOnPause
	 */
	this._unbindOnPause = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PAUSE, self._triggerOnPause);
	};
    
    /**
	 * @method _triggerOnPause
	 */
    this._triggerOnPause = function() {
        for (var i = 0; i < self.list.ON_PAUSE.length; i++) {
			if (self.list.ON_PAUSE[i]) self.list.ON_PAUSE[i](mp);
		}
    };

	/**
	 * @method _bindOnEnded
	 */
	this._bindOnEnded = function() {
		mp.player.engine.on(Clappr.Events.PLAYER_ENDED, self._triggerOnEnded);
	};
    
    /**
	 * @method _unbindOnEnded
	 */
	this._unbindOnEnded = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_ENDED, self._triggerOnEnded);
	};
    
    /**
     * @method _triggerOnEnded
     */
    this._triggerOnEnded = function() {
        for (var i = 0; i < self.list.ON_ENDED.length; i++) {
			if (self.list.ON_ENDED[i]) self.list.ON_ENDED[i](mp);
		}
    };
    
    /**
     * @method _bindOnMcRendered
     */
    this._bindOnMcRendered = function() {
        mp.player.engine.listenToOnce(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcRendered);
    };
    
    /**
     * @method _unbindOnMcRendered
     */
    this._unbindOnMcRendered = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcRendered);
    };
    
    /**
     * @method _triggerOnMcRendered
     */
    this._triggerOnMcRendered = function() {
        for (var i = 0; i < self.list.ON_MC_RENDERED.length; i++) {
			if (self.list.ON_MC_RENDERED[i]) self.list.ON_MC_RENDERED[i](mp);
		}
    };
    
    /**
	 * @method _bindOnMcHide
	 */
    this._bindOnMcHide = function() {
        mp.player.engine.listenTo(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_HIDE, self._triggerOnMcHide);
    };
    
    /**
	 * @method _unbindOnMcHide
	 */
    this._unbindOnMcHide = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_HIDE, self._triggerOnMcHide);
    };
    
    /**
     * @method _triggerOnMcHide
     */
    this._triggerOnMcHide = function() {
        for (var i = 0, l = self.list.ON_MC_HIDE.length; i < l; i++) {
			if (self.list.ON_MC_HIDE[i]) self.list.ON_MC_HIDE[i](mp);
		}
    };
    
    /**
	 * @method _bindOnMcShow
	 */
    this._bindOnMcShow = function() {
        mp.player.engine.listenTo(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcShow);
    };
    
    /**
	 * @method _unbindOnMcShow
	 */
    this._unbindOnMcShow = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcShow);
    };
    
    /**
     * @method _triggerOnMcShow
     */
    this._triggerOnMcShow = function() {
        for (var i = 0, l = self.list.ON_MC_SHOW.length; i < l; i++) {
			if (self.list.ON_MC_SHOW[i]) self.list.ON_MC_SHOW[i](mp);
		}
    };
    
    /**
     * @method _triggerOnCreated
     */
    this._triggerOnCreated = function() {
        for (var i = 0; i < self.list.ON_CREATED.length; i++) {
			if (self.list.ON_CREATED[i]) self.list.ON_CREATED[i](mp);
		}
    };
    
    /**
     * @method _triggerOnRefresh
     */
    this._triggerOnRefresh = function() {
        for (var i = 0; i < self.list.ON_REFRESH.length; i++) {
			if (self.list.ON_REFRESH[i]) self.list.ON_REFRESH[i](mp);
		}
    };
    
    /**
	 * @method _triggerPreventPlay
	 */
	this._triggerPreventPlay = function(e) {
        if (self.list.PREVENT_PLAY[0] && !this.keepPreventingEventsDiabled) {
            e.stopPropagation();
            for (var i = 0; i < self.list.PREVENT_PLAY.length; i++) {
            	if (self.list.PREVENT_PLAY[i]) self.list.PREVENT_PLAY[i](mp);
            }
        }
    };

	/**
	 * @method bind
	 */
	this.bind = function() {
        this._bindOnPlayIntent();
		this._bindOnLoadMetadata();
		this._bindOnFirstClickToPlay();
		this._bindOnFirstPlay();
        this._bindOnFirstPause();
        this._bindOnPlay();
		this._bindOnPause();
		this._bindOnEnded();
        this._bindOnMcRendered();
        this._bindOnMcHide();
        this._bindOnMcShow();
        
        this._triggerOnCreated();
	};
    
    /**
     * @method refresh
     */
    this.refresh = function() {
        this._unbindOnPlayIntent();
        this._unbindOnLoadMetadata();
        this._unbindOnFirstClickToPlay();
        this._unbindOnFirstPlay();
        this._unbindOnFirstPause();
        this._unbindOnPlay();
        this._unbindOnEnded();
        this._unbindOnMcRendered();
        this._unbindOnMcHide();
        this._unbindOnMcShow();
        
        this._bindOnPlayIntent();
        this._bindOnLoadMetadata();
        this._bindOnFirstClickToPlay();
        this._bindOnFirstPlay();
        this._bindOnFirstPause();
        this._bindOnPlay();
        this._bindOnEnded();
        this._bindOnMcRendered();
        this._bindOnMcHide();
        this._bindOnMcShow();
        
        this._triggerOnRefresh();
    };
    
	this.list.ON_CREATED = [];
    this.list.ON_REFRESH = [];
    this.list.ON_PLAY_INTENT = [];
	this.list.ON_LOAD_METADATA = [];
	this.list.ON_FIRST_CLICK_TO_PLAY = [];
	this.list.ON_FIRST_PLAY = [];
    this.list.ON_FIRST_PAUSE = [];
    this.list.ON_PLAY = [];
	this.list.ON_PAUSE = [];
	this.list.ON_ENDED = [];
    this.list.ON_MC_RENDERED = [];
    this.list.ON_MC_HIDE = [];
    this.list.ON_MC_SHOW = [];
	this.list.PREVENT_PLAY = [];
}


/**
 * @class MoonplayerExecute
 *
 * @param {Object} mp Object of Moonplayer Class
 * @method script Execute string as a javascript code at the document head
 */
function MoonplayerExecute(mp) {

    /**
     * @method script
     *
     * @param {String} script Strint to execute
     * @param {Number|Boolean} timeToRemove Milliseconds to remove script tag (pass a false to not remove)
     */
	this.script = function(string, timeToRemove) {

		if (typeof string !== 'string') return false;

		var elementScript = document.createElement('script');
		elementScript.innerHTML = string;

		var documentHead = document.querySelector('head');

		documentHead.appendChild(elementScript);

		if (timeToRemove !== false) {
			setTimeout(function() {
				documentHead.removeChild(elementScript);
			}, typeof timeToRemove === 'number' ? timeToRemove : 1000);
        }
        
	};

}


/**
 * @class MoonplayerPlaybar
 *
 * @param {Object} mp Object of Moonplayer Class
 * @property {Object} el DOMElement #playbar
 * @method _genButtonElement Generate playbar button from array list
 * @method flush Create or update playbar elements
 */
function MoonplayerPlaybar(mp) {

	var self = this;

	this.el = null;

	/**
	 * @method _genButtonElement
	 *
	 * @param {Array} listArray Important value! Array of entity with keys: name, url, selected
	 * @param {String} arrayName Important value! The class of button element
	 * @param {String} buttonLabel Important value! The label of button element (pattern %name%)
	 * @param {String} itemLabel The label of list item (pattern %name%)
	 * @param {String} noLabel Alternative name for a not existing name value of list item (pattern %index%)
	 * @param {Function} onClick The Callback function on click event for each anchor elements
	 * @param {Number} nameLength Maximum name characters for split by space
	 */
	this._genButtonElement = function(listArray, arrayName, buttonLabel, itemLabel, noLabel, onClick, nameLength) {
		var documentButton = document.createElement('div'),
			documentLabel = document.createElement('span'),
			documentList = document.createElement('ul');

		if (!listArray.length) documentButton.style.display = 'none';
		documentButton.className = 'playbar-button playbar-button-' + arrayName;
		documentLabel.innerHTML = buttonLabel.replace(/\s?%name%\s?/g, '');

		for (var i = 0, l = listArray.length; i < l; i++) {
			var documentAnchor = document.createElement('a');
			var documentItem = document.createElement('li');

			// anchor event
			if (onClick) documentAnchor.addEventListener('click', onClick);

			// anchor name
			documentAnchor.innerHTML = itemLabel && listArray[i].name && /%name%/.test(itemLabel) ?
				itemLabel.replace(/%name%/g, listArray[i].name) : listArray[i].name ?
				listArray[i].name : /%index%/.test(noLabel) ?
				noLabel.replace(/%index%/g, i + 1) : noLabel || 'undefined';
            
            // anchor name length
            if (nameLength && documentAnchor.innerHTML[nameLength])
                documentAnchor.innerHTML = documentAnchor.innerHTML.split(' ')[0];
            
			// url
			if (listArray[i].url) documentAnchor.setAttribute('href', listArray[i].url);

			// selected
			if (listArray[i].selected) {
				documentItem.className = 'selected';
				documentLabel.innerHTML = listArray[i].name && /%name%/.test(buttonLabel) ?
					buttonLabel.replace(/%name%/g, listArray[i].name) : buttonLabel;
			}

			documentItem.appendChild(documentAnchor);
			documentList.appendChild(documentItem);
		}

		documentButton.appendChild(documentLabel);
		documentButton.appendChild(documentList);

		return documentButton;
	};

	/**
	 * @method flush
	 */
	this.flush = function() {

		if (!this.el) {
			this.el = document.createElement('div');
			this.el.setAttribute('id', 'playbar');
			mp.documentPlayer.appendChild(this.el);
			// events: show or hide with mediacontrol
			mp.events.list.ON_MC_HIDE.push(function() {
				self.el.className = 'hide';
			});
			mp.events.list.ON_MC_SHOW.push(function() {
				self.el.className = '';
			});
            // events: force to show after ended
			mp.events.list.ON_ENDED.push(function() {
				self.el.className = '';
                mp.player.engine.core.mediaControl.keepVisible = true;
			});
            // events: force to show mediacontrol on hover on playbar
			this.el.addEventListener('mouseenter', function() {
				mp.player.engine.core.mediaControl.show();
				mp.player.engine.core.mediaControl.keepVisible = true;
			});
			this.el.addEventListener('mouseleave', function() {
				if (mp.player.engine.isPlaying()) mp.player.engine.core.mediaControl.keepVisible = false;
			});
		}

		this.el.innerHTML = '';
        this.el.className = '';
        
        var arrowHTML = '<svg xmlns="http://www.w3.org/2000/svg" height="9" width="11" viewBox="0 0 11 9"><g transform="matrix(.027363 0 0 .024322 2 .50211)"><path d="m286.94 70.277c-3.614-3.617-7.898-5.424-12.848-5.424h-255.81c-4.952 0-9.233 1.807-12.85 5.424-3.617 3.621-5.424 7.902-5.424 12.851 0 4.948 1.807 9.229 5.424 12.847l127.9 127.9c3.621 3.617 7.902 5.428 12.85 5.428s9.233-1.811 12.847-5.428l127.91-127.91c3.613-3.617 5.427-7.898 5.427-12.847 0-4.948-1.814-9.229-5.427-12.85z"/></g></svg>';

        // enconde cyrillic characters
        var seasonCyr  = decodeURIComponent('%D1%81%D0%B5%D0%B7%D0%BE%D0%BD');
        var episodeCyr = decodeURIComponent('%D1%81%D0%B5%D1%80%D0%B8%D1%8F');
        var releaseCyr = decodeURIComponent('%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4');
        var emptyCyr   = decodeURIComponent('%D0%B1%D0%B5%D0%B7 %D0%B8%D0%BC%D0%B5%D0%BD%D0%B8');

		this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.seasons,
			'seasons',
            seasonCyr +' %name% '+ arrowHTML,
            seasonCyr +' %name%',
            emptyCyr,
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            }
		));

		this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.episodes,
			'episodes',
            episodeCyr +' %name% '+ arrowHTML,
            episodeCyr +' %name%',
            emptyCyr,
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            }
		));

		if (mp.options.releasesEnabled) this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.releases,
			'releases',
            releaseCyr +' '+ arrowHTML,
			'%name%',
            releaseCyr +' %index%',
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            },
            14
		));

        // enconde cyrillic characters
		var playerCyr = decodeURIComponent('%D0%9F%D0%BB%D0%B5%D0%B5%D1%80');
        var linkCyr   = decodeURIComponent('%D0%A1%D1%81%D1%8B%D0%BB%D0%BA%D0%B0');
		var codeCyr   = decodeURIComponent('%D0%BA%D0%BE%D0%B4 %D0%B2%D0%B8%D0%B4%D0%B5%D0%BE');

		if (mp.options.playerCodeEnabled) this.el.appendChild(this._genButtonElement(
			[{
				name: playerCyr +':<br><textarea><iframe src="http://moonwalk.cc' + location.pathname.split('?')[0].replace(mp.options.scriptPath, '') + '" width="610" height="370" frameborder="0" allowfullscreen></iframe></textarea>'
			}, {
				name: linkCyr +':<br><textarea>http://moonwalk.cc' + location.pathname.split('?')[0].replace(mp.options.scriptPath, '') + '</textarea>'
			}],
			'code',
            codeCyr
		));

	};

}


/**
 * @class MoonplayerPlayer
 *
 * @property {Object} engine            Object of Clappr.Player Class
 * @property {Object} elementContainer  DOMElement #player .container
 * @method create                       Initialize Clappr with source from MoonplayerData Class
 * @method updateSource                 To load new source from MoonplayerData Class
 * @method _isFlash                     Return true if engine uses flash
 * @method buffering                    To try to start buffering
 * @method showMessage                  To show player message
 * @method hideMessage                  To hide player message
 */
function MoonplayerPlayer(mp) {

    var self = this;

	this.engine = null;
	this.elementContainer = null;

	/**
	 * @method create
	 */
	this.create = function() {

		this.engine = new Clappr.Player({
			width: '100%',
			height: '100%',
			parent: mp.documentPlayer,
			source: mp.data.json.manifest,
			baseUrl: mp.options.assetsPath,
			autoplay: false,
			preload: 'metadata',
			poster: mp.options.posterUrl ? mp.options.posterUrl : mp.data.json.poster ? mp.data.json.poster : undefined,
			plugins: {
				core: [LevelSelector, ClapprSubtitle],
				playback: mp.options.flashEnabled ? [Clappr.FlasHLS] : []
			},
			autoSeekFromUrl: false,
			exitFullscreenOnEnd: false,
			levelSelectorConfig: {
				labels: {
					3: '1080p',
					2: '720p',
					1: '480p',
					0: '360p'
				}
			},
			playbackNotSupportedMessage: mp.options.notSupportedMsg,
            subtitle: ''
		});

		this.elementContainer = this.engine.core.el.querySelector('.container');

	};
    
    /**
     * @method updateSource
     */
    this.updateSource = function() {
        if (self.engine) self.engine.load({source: mp.data.json.manifest, mimeType: 'application/x-mpegURL'});
    };
    
    /**
     * @method _isFlash
     *
     * @returns {Boolean|Null} Return true if engine uses flash
     */
    this._isFlash = function() {
        return self.engine ? !!self.engine.getPlugin('flashls') : null;
    };
    
    /**
     * @method buffering
     */
    this.buffering = function() {
        if (self.engine && !self._isFlash()) {
            self.engine.play();
            self.engine.pause();
        } else if (self.engine) {
            self.engine.play();
            self.engine.core.getCurrentContainer().once(
                Clappr.Events.CONTAINER_LOADEDMETADATA,
                function() {
                    self.engine.pause();
                }
            );
        }
    };
    
    /**
     * @method showMessage
     *
     * @param {String} message      Message to show
     * @param {Boolean} autohide    Hide message automatically
     */
    this.showMessage = function(message, autohide) {
        var seekTimePlugin = mp.player.engine.core.getPlugin('seek_time');
        if (!seekTimePlugin.hoverPosition) seekTimePlugin.hoverPosition = 0;
        seekTimePlugin.hoveringOverSeekBar = true;
        seekTimePlugin.update();
        seekTimePlugin.$seekTimeEl[0].innerHTML = message;
        if (autohide) this.hideMessage(1000);
    };
    
    /**
     * @method hideMessage
     *
     * @param {Number} time     Time (ms) to hide message
     */
    this.hideMessage = function(time) {
        var seekTimePlugin = mp.player.engine.core.getPlugin('seek_time');
        setTimeout(function() {
            seekTimePlugin.hoveringOverSeekBar = false;
            seekTimePlugin.update();
        }, time || 1);
    };

}


/**
 * @class MoonplayerPlaylist
 *
 * @param {Object} mp Object of Moonplayer Class
 * @method _getCurrentIndexes Returns object contain indexes by season, spisode and release keys
 * @method getCurrentNames Returns names of current season, episode and release
 * @method getNextNames Returns names of next season and episode with current release
 * @method getUrlByNames Returns url string by season and episode name
 */
function MoonplayerPlaylist(mp) {

    /**
     * @method _getCurrentIndexes
     *
     * @returns {Object} Object contain indexes by season, spisode and release keys
     */
    this._getCurrentIndexes = function() {
        var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
			indexSeason = null,
			indexEpisode = null,
			indexRelease = null;
        for (var i = 0, l = seasons.length; i < l; i++) {
			if (seasons[i].selected) indexSeason = i;
		}
		for (var i = 0, l = episodes.length; i < l; i++) {
			if (episodes[i].selected) indexEpisode = i;
		}
		for (var i = 0, l = releases.length; i < l; i++) {
			if (releases[i].selected) indexRelease = i;
		}
        return {
			season: indexSeason,
			episode: indexEpisode,
            release: indexRelease
		};
    };

    /**
	 * @method getCurrentNames
	 *
	 * @returns {Object} Object contain names by season, episode and release keys
	 */
	this.getCurrentNames = function() {
		var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
            indexes = this._getCurrentIndexes();
		return {
			season: indexes.season !== null ? seasons[indexes.season].name : null,
			episode: indexes.episode !== null ? episodes[indexes.episode].name : null,
            release: indexes.release !== null ? releases[indexes.release].name : null
		};
	};
    
    /**
     * @method getNextNames
     *
     * @returns {Object} Object contain names by season, episode and release keys
     */
    this.getNextNames = function() {
        var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
            indexes = this._getCurrentIndexes(),
            namesNext = Object.create(null);
        
        namesNext.season =
        namesNext.episode =
        namesNext.release = null;
        
        // current release
        if (indexes.release !== null && indexes.release in releases)
            namesNext.release = releases[indexes.release].name;
        
        // season is not exists
        if (indexes.season === null) return namesNext;
        
        // episode is exist and not is latest
        if (indexes.episode !== null && indexes.episode +1 in episodes) {
            namesNext.season = seasons[indexes.season].name;
            namesNext.episode = episodes[indexes.episode +1].name;
            return namesNext;
        }
        
        // episode is latest
        namesNext.season = indexes.season +1 in seasons ? seasons[indexes.season +1].name : seasons[0].name;
        return namesNext;
    };

	/**
	 * @method getUrlByNames
	 *
	 * @param {String} Name of season
	 * @param {String} Name of episode
	 * @returns {String|null} Url
	 */
	this.getUrlByNames = function(nameSeason, nameEpisode) {
		var url = null,
			seasons = mp.data.json.playlist.seasons;
		
        if (nameSeason === null) return null;
		
        for (var i = 0, l = seasons.length; i < l; i++) {

			if (seasons[i].name === nameSeason) {

                url = seasons[i].url;
                break;
            }
		}
        
        if (url && nameEpisode !== null)
            url = url.replace(/episode=(.*?)[^&]/, 'episode='+ nameEpisode);
        else if (url)
            url = url.replace(/episode=(.*?)[^&]/, 'episode=1');
        
		return url;
	};

}


/**
 * @class MoonplayerPresets
 *
 * @method fixFirefoxComutedStyle The Firefox iframe bug fixing
 * @method rewriteFullscreen Rewrite a Clappr fullscreen methods
 * @method setMediaControlTemplate Set a Clappr MediaControl template
 * @method showMediaControlOnPause Force to show MediaControl on pause
 * @method preventClickAfterPlayIntentOnFlash Prevent click after play intent and before first play (FlasHLS)
 */
function MoonplayerPresets(mp) {

	/**
	 * @method fixFirefoxComutedStyle
	 */
	this.fixFirefoxComutedStyle = function() {
		if (/firefox/i.test(navigator.userAgent)) {
			window.oldGetComputedStyle = window.getComputedStyle;
			window.getComputedStyle = function(t, e) {
				var o = window.oldGetComputedStyle(t, e);
				return null === o ? {
					getPropertyValue: function() {}
				} : o;
			};
		}
	};
    
    /**
	 * @method rewriteFullscreen
	 */
	this.rewriteFullscreen = function() {
        var elementContainer,
            documentParent,
            isAccessToParent;
            
        try {
            elementContainer = window.frameElement;
            documentParent = window.parent.document;
            isAccessToParent = true;
        } catch (e) { }
        
        if (elementContainer && documentParent) {
            isAccessToParent = true;
        } else {
            elementContainer = mp.documentPlayer;
            documentParent = document;
            isAccessToParent = false;
        }
        
		Clappr.Utils.Fullscreen.isFullscreen = function() {
			return !!(documentParent.webkitFullscreenElement || documentParent.webkitIsFullScreen || documentParent.mozFullScreen || documentParent.msFullscreenElement || document.webkitFullscreenElement || document.webkitIsFullScreen);
		};
		Clappr.Utils.Fullscreen.requestFullscreen = function(el) {
			if (elementContainer.requestFullscreen) elementContainer.requestFullscreen();
			else if (elementContainer.webkitRequestFullscreen) elementContainer.webkitRequestFullscreen();
			else if (elementContainer.mozRequestFullScreen) elementContainer.mozRequestFullScreen();
			else if (elementContainer.msRequestFullscreen) elementContainer.msRequestFullscreen();
			else if (el.querySelector && el.querySelector("video") && el.querySelector("video").webkitEnterFullScreen) el.querySelector("video").webkitEnterFullScreen();
		};
		Clappr.Utils.Fullscreen.cancelFullscreen = function() {
			if (documentParent.exitFullscreen) documentParent.exitFullscreen();
			else if (documentParent.webkitCancelFullScreen) documentParent.webkitCancelFullScreen();
			else if (documentParent.webkitExitFullscreen) documentParent.webkitExitFullscreen();
			else if (documentParent.mozCancelFullScreen) documentParent.mozCancelFullScreen();
			else if (documentParent.msExitFullscreen) documentParent.msExitFullscreen();
		};
        
        if (!isAccessToParent) document.addEventListener('keyup', function(e) {
			if (mp.player.engine && (e.which === 27 || e.which === 27 || e.keyCode === 27)) {
				mp.player.engine.core.mediaControl.toggleFullscreen();
				mp.player.engine.core.mediaControl.toggleFullscreen();
			}
		});
        
	};

	/**
	 * @method setMediaControlTemplate
	 */
	this.setMediaControlTemplate = function() {
		var clapprTemplate = Object.getOwnPropertyDescriptor(Clappr.MediaControl.prototype, 'template');
		clapprTemplate.get = function() {
			return Clappr.template('<div class="media-control-background" data-background></div> <div class="media-control-layer" data-controls> <% var renderBar = function(name) { %> <div class="bar-container" data-<%= name %>> <div class="bar-background" data-<%= name %>> <div class="bar-fill-1" data-<%= name %>></div> <div class="bar-fill-2" data-<%= name %>></div> <div class="bar-hover" data-<%= name %>></div> </div> <div class="bar-scrubber" data-<%= name %>> <div class="bar-scrubber-icon" data-<%= name %>></div> </div> </div> <% }; %> <% var renderSegmentedBar = function(name, segments) { segments = segments || 10; %> <div class="bar-container" data-<%= name %>> <% for (var i = 0; i < segments; i++) { %> <div class="segmented-bar-element" data-<%= name %>></div> <% } %> </div> <% }; %> <% var renderDrawer = function(name, renderContent) { %> <div class="drawer-container" data-<%= name %>> <div class="drawer-icon-container" data-<%= name %>> <div class="drawer-icon media-control-icon" data-<%= name %>></div> <span class="drawer-text" data-<%= name %>></span> </div> <% renderContent(name); %> </div> <% }; %> <% var renderIndicator = function(name) { %> <div class="media-control-indicator" data-<%= name %>></div> <% }; %> <% var renderButton = function(name) { %> <button class="media-control-button media-control-icon" data-<%= name %>></button> <% }; %> <% var templates = { bar: renderBar, segmentedBar: renderSegmentedBar, }; var render = function(settingsList) { settingsList.forEach(function(setting) { if(setting === "seekbar") { renderBar(setting); } else if (setting === "volume") { renderDrawer(setting, settings.volumeBarTemplate ? templates[settings.volumeBarTemplate] : function(name) { return renderSegmentedBar(name); }); } else if (setting === "duration" || setting === "position") { renderIndicator(setting); } else { renderButton(setting); } }); }; %> <% if (settings.left && settings.left.length) { %> <div class="media-control-left-panel" data-media-control> <% render(settings.left); %> </div> <% } %> <% if (settings.right && settings.right.length) { %> <div class="media-control-right-panel" data-media-control> <% render(settings.right); %> </div> <% } %> <% if (settings.default && settings.default.length) { %> <div class="media-control-center-panel" data-media-control> <% render(settings.default); %> </div> <% } %> </div>');
		};
		Object.defineProperty(Clappr.MediaControl.prototype, 'template', clapprTemplate);
	};
    
    /**
     * @method showMediaControlOnPause
     */
    this.showMediaControlOnPause = function() {
        mp.events.list.ON_PAUSE.push(function() {
            mp.player.engine.core.mediaControl.keepVisible = true;
        });
        mp.events.list.ON_PLAY.push(function() {
            mp.player.engine.core.mediaControl.keepVisible = false;
        });
    };
    
    /**
     * @method preventClickAfterPlayIntentOnFlash
     */
    this.preventClickAfterPlayIntentOnFlash = function() {
        
        function prevent(e) {
            e.stopPropagation();
            e.preventDefault();
            return false;
        }
        
        mp.events.list.ON_CREATED.push(function() {
        
            if (mp.player._isFlash()) {
            
                mp.events.list.ON_PLAY_INTENT.push(function() {
                    mp.documentPlayer.addEventListener('click', prevent, true);
                });
                
                mp.events.list.ON_FIRST_PLAY.push(function() {
                    mp.documentPlayer.removeEventListener('click', prevent, true);
                });
                
                // fix for prevented play
                mp.events.list.ON_FIRST_PAUSE.push(function() {
                    mp.documentPlayer.removeEventListener('click', prevent, true);
                });
            
            }
            
        });
        
    };

}


/**
 * @class MoonplayerRollback
 *
 * @param {Object} mp Objec of Moonplayer Class
 * @property {Object} el DOMElement .rollback-bar
 * @method _getNamesFromCookie Return object contain names by season and episode key from cookie
 * @method setCurrentCookie To set 'mp_rollback' cookie by the current season and episode names
 * @method show To create and show a Rollback element
 * @method hide To remove a Rollback element
 */
function MoonplayerRollback(mp) {

    this.el = null;
    
    /**
     * @method _getNamesFromCookie
     *
     * @returns {Object|null} Return object contain names by season and episode key
     */
    this._getNamesFromCookie = function() {
        if (document.cookie.indexOf('mp_rollback=') < 0) return null;
        var names = /mp_rollback=([^;\s]*)/.exec(document.cookie)[1].split('+');
        if (!names[0] || !names[1]) return null;
        return {season: names[0], episode: names[1]};
    };
    
    /**
     * @method setCurrentCookie
     */
    this.setCurrentCookie = function() {
        var names = mp.playlist.getCurrentNames(),
            expires = new Date();
        expires.setDate(expires.getDate() + 14);
        if (!names.season || !names.episode) return false;
        document.cookie = 'mp_rollback=' + names.season + '+' + names.episode + '; expires=' + expires.toUTCString();
    };
    
    /**
     * @method show
     */
    this.show = function() {
        // get names
        var namesCookie = this._getNamesFromCookie(),
            namesCurrent = mp.playlist.getCurrentNames();
        // return if names not defined
        if (!namesCookie || !namesCurrent.season || !namesCurrent.episode) return false;
        // return if names is equal
        if (namesCookie.season === namesCurrent.season && namesCookie.episode === namesCurrent.episode) return false;
        // get url by season and episode from cookie
        var url = mp.playlist.getUrlByNames(namesCookie.season, namesCookie.episode);
        // return if url is not defined
        if (!url) return false;
        // create rollback element if not exists
        if (!this.el) {
            this.el = document.createElement('div');
            this.el.className = 'rollback-bar';
        }
        this.el.innerHTML =
            decodeURIComponent('%D0%92%D1%8B %D0%BE%D1%81%D1%82%D0%B0%D0%BD%D0%BE%D0%B2%D0%B8%D0%BB%D0%B8%D1%81%D1%8C %D0%BD%D0%B0 ')
            + namesCookie.episode
            + decodeURIComponent(' %D1%81%D0%B5%D1%80%D0%B8%D0%B8 ')
            + namesCookie.season
            + decodeURIComponent(' %D1%81%D0%B5%D0%B7%D0%BE%D0%BD%D0%B0 ')
            + '<a href="' + url + '">'
            + decodeURIComponent('%D0%BF%D0%B5%D1%80%D0%B5%D0%B9%D1%82%D0%B8%C2%BB')
            + '</a>';

        mp.documentLayer.appendChild(this.el);
    };
    
    /**
     * @method hide
     */
    this.hide = function() {
        if (this.el && this.el.parentNode) this.el.parentNode.removeChild(this.el);
    };
    
}


/**
 * @class MoonplayerSpinner
 *
 * @param {Object} mp Objec of Moonplayer Class
 * @property {Object} el DOMElement #spinner
 * @method show Show spinner element
 * @method hide Hide spinner element
 */
function MoonplayerSpinner(mp) {

	this.el = null;

	/**
	 * @method show
	 */
	this.show = function() {
		if (!this.el) {
			this.el = document.createElement('div');
			this.el.setAttribute('id', 'spinner');
			this.el.innerHTML = '<div data-spinner="" class="spinner-three-bounce"><div data-bounce1=""></div><div data-bounce2=""></div><div data-bounce3=""></div></div>';
		}
		mp.documentLayer.appendChild(this.el);
	};

	/**
	 * @method hide
	 */
	this.hide = function() {
		if (mp.documentLayer.contains(this.el))
            mp.documentLayer.removeChild(this.el);
	};

}


/**
 * @class Moonplayer
 *
 * @property {Object} options           Object of Settings Data
 * @property {Object} documentLayer     DOMElement #layer
 * @property {Object} documentPlayer    DOMElement #player
 * @property {Object} presets           Object of MoonplayerPresets Class
 * @property {Object} data              Object of MoonplayerData Class
 * @property {Object} spinner           Object of MoonplayerData Class
 * @property {Object} player            Object of MoonplayerPlayer Class
 * @property {Object} events            Object of MoonplayerEvents Class
 * @property {Object} playbar           Object of MoonplayerPlaybar Class
 * @property {Object} playlist          Object of MoonplayerPlaylist Class
 * @property {Object} rollback          Object of MoonplayerRollback Class
 * @property {Object} autoplay          Object of MoonplayerAutoplay Class
 * @property {Object} banner            Object of MoonplayerBanner Class
 * @property {Object} execute           Object of MoonplayerExecute Calss
 * @method init                         Initialize Moonplayer
 * @method load                         To load source by iframe url (ajax)
 * @method error                        Thrown exception
 * @method warning                      Thrown warning
 * @method isMobile                     Returns true if mobile browser was detected
 */
function Moonplayer() {

	var self = this;

	this.options = {};

	this.documentLayer = document.getElementById('layer');
	this.documentPlayer = document.getElementById('player');

	this.presets = new MoonplayerPresets(this);
	this.data = new MoonplayerData(this);
	this.spinner = new MoonplayerSpinner(this);
	this.player = new MoonplayerPlayer(this);
	this.events = new MoonplayerEvents(this);
	this.playbar = new MoonplayerPlaybar(this);
    this.playlist = new MoonplayerPlaylist(this);
	this.rollback = new MoonplayerRollback(this);
	this.autoplay = new MoonplayerAutoplay(this);
    this.banner = new MoonplayerBanner(this);
    this.execute = new MoonplayerExecute(this);

	/**
	 * @method init
	 */
	this.init = function() {
		// presets
		this.presets.fixFirefoxComutedStyle();
		this.presets.rewriteFullscreen();
		this.presets.setMediaControlTemplate();
        this.presets.showMediaControlOnPause();
        this.presets.preventClickAfterPlayIntentOnFlash();
		// show spinner
		this.spinner.show();
		// query
		this.data.load(this.options.parserData, function(/*json*/) {

            /*
            self.events.list.ON_CREATED.push(function() { console.log('ON_CREATED'); });
			self.events.list.ON_REFRESH.push(function() { console.log('ON_REFRESH'); });
            self.events.list.ON_PLAY_INTENT.push(function() { console.log('ON_PLAY_INTENT'); });
			self.events.list.ON_LOAD_METADATA.push(function() { console.log('ON_LOAD_METADATA'); });
			self.events.list.ON_FIRST_CLICK_TO_PLAY.push(function() { console.log('ON_FIRST_CLICK_TO_PLAY'); });
			self.events.list.ON_FIRST_PLAY.push(function() { console.log('ON_FIRST_PLAY'); });
            self.events.list.ON_PLAY.push(function() { console.log('ON_PLAY'); });
			self.events.list.ON_PAUSE.push(function() { console.log('ON_PAUSE'); });
            self.events.list.ON_ENDED.push(function() { console.log('ON_ENDED'); });
            self.events.list.ON_MC_RENDERED.push(function() { console.log('ON_MC_RENDERED'); });
            self.events.list.ON_MC_HIDE.push(function() { console.log('ON_MC_HIDE'); });
			self.events.list.ON_MC_SHOW.push(function() { console.log('ON_MC_SHOW'); });
			self.events.list.PREVENT_PLAY.push(function() { console.log('PREVENT_PLAY'); self.player.engine.play(); });
            */
            
			// rollback
			if (self.options.rollbackEnabled) {
				self.rollback.show();
				self.events.list.ON_FIRST_PLAY.push(function() {
					self.rollback.hide();
					self.rollback.setCurrentCookie();
				});
			}

			// autoplay
            if (self.options.autoplayEnabled && !self.isMobile()) {
                self.events.list.ON_MC_RENDERED.push(function() {
                    self.autoplay.flush();
                });
                self.events.list.ON_ENDED.push(function() {
                    self.autoplay.next();
                });
            }
            
            // force to hide spinner
            self.events.list.ON_LOAD_METADATA.push(function() {
                self.spinner.hide();
            });
            
			self.player.create();
			self.events.bind();
			self.playbar.flush();
			self.spinner.hide();
		});
	};

	/**
	 * @method load
	 */
	this.load = function(url, autoplay) {
		// show spinner
		this.spinner.show();
        // stop playing
        if (self.player.engine.isPlaying()) self.player.engine.pause();
		// query
		this.data.load(this.data.urlToData(url), function(/*json*/) {
			self.player.updateSource();
			self.events.refresh();
			self.playbar.flush();
            if (autoplay) self.player.engine.play();
            else self.spinner.hide()
		});
	};

	/**
	 * @method error
	 */
	this.error = function(msg) {
		location.href = this.options.realUrl;
		throw msg || '[Moonplayer] unknown error';
	};
    
    /**
     * @method warning
     */
    this.warning = function(msg) {
        console.warn(msg || '[Moonplayer] unknown warning');
    };
    
    /**
     * @method isMobile
     *
     * @returns {Boolean} Returns true if mobile browser was detected
     */
    this.isMobile = function() {
        return /android|blackberry|iphone|ipad|ipod|opera mini|iemobile/i.test(navigator.userAgent);
    };
}


