/**
 * @class MoonplayerBanner
 *
 * @param {Object} mp           Object of Moonplayer Class
 * @method _normalize           Validate and normalize defined options
 * @method _centralizeStyle     To set a centralized styles to element
 * @method _genBannerElements   Generate banner elements by defined options
 * @method _setAction           To define banner actions 
 * @method show                 Create and show the banner
 */
function MoonplayerBanner(mp) {

    /**
     * To validate and normalize list of values:
     * script {String}      String of javascript to execute on head element (default: null)
     * content {String}     Banner content (default: null) also you can pass element id to use the inner content
     * closeText {String}   Value of close button (default: закрыть)
     * timerText {String}   Value of close button if timer is running (pattern %count% will be replaced at seconds) (default: осталось %count% сек.)
     * timer {Number}       Seconds of timer delay (default: 0)
     * assistText {String}  Auxiliary message on banner (default: null)
     * width {Number}       The width property in px (default: 200)
     * height {Number}      The height property in px (default: 200)
     * filled {Boolean}     Hide background when banner is showing (default: true)
     * callback {Function}  This function will be called after the banner will be closed (default: null)
     *
     * @method _normalize
     * @param {Object} opt  Defined banner options
     */
    this._normalize = function(opt) {
        
        if (!opt) return false;
        
        opt.script = typeof opt.script === 'string' && opt.script.length ? opt.script : null;
        opt.content = typeof opt.content === 'string' && opt.content.length ? opt.content : null;
        
        if (opt.content && opt.content.indexOf('#') === 0) {
            var elementContent = document.getElementById(opt.content.substr(1));
            opt.content = elementContent ? elementContent.innerHTML : null;
        }
        
        if (!opt.script && !opt.content) return false;
        
        opt.closeText = opt.closeText || 'закрыть';
        opt.timerText = opt.timerText || 'осталось %count% сек.';
        opt.assistText = opt.assistText || null;
        
        opt.timer = typeof opt.timer === 'number' && opt.timer ? opt.timer : 0;
        
        opt.width = parseInt(opt.width || 200);
        opt.height = parseInt(opt.height || 200);
        
        opt.filled = !!opt.filled;
        
        opt.callback = typeof opt.callback === 'function' ? opt.callback : null;
        
        return opt;
        
    };
    
    /**
     * @method _centralizeStyle
     *
     * @param {Object} element Element to set centralized styles
     * @param {Number} width   The width of element in px (require for calculation)
     * @param {Number} height  The height of element in px (require for calculation)
     */
    this._centralizeStyle = function(element, width, height) {
        
        if (!element) return false;
        
        if (width || height) element.style.position = 'absolute';
        
        if (width) {
            element.style.top = '50%';
            element.style['margin-left'] = -width/2 +'px';
        }
        
        if (height) {
            element.style.left = '50%';
            element.style['margin-top'] = -height/2 +'px';
        }
        
    };
    
    /**
     * @method _genBannerElements
     *
     * @param {Object} options Defined banner options
     * @returns {Object} Associated array with elements
     */
    this._genBannerElements = function(options) {
        
        // script element
        var elementScript = null;
        
        if (options.script) {
            elementScript = document.createElement('script');
            elementScript.innerHTML = options.script;
        }
        
        // content element
        var elementContent = null;
        
        if (options.content) {
            elementContent = document.createElement('div');
            elementContent.className = 'banner-content';
            elementContent.innerHTML = options.content;
        }
        
        // assist element
        var elementAssist = null;
        
        if (options.assistText) {
            elementAssist = document.createElement('div');
            elementAssist.className = 'banner-assist';
            elementAssist.innerHTML = options.assistText;
        }
        
        // root element
        var elementBanner = document.createElement('div');
        elementBanner.className = 'banner';
        elementBanner.className += options.filled ? ' banner-filled' : '';
        
        // close element
        var elementClose = document.createElement('div');
        elementClose.className = 'banner-close';
        
        // the timer element is a child of close element
        var elementTimer = null;
        
        if (options.timer) {
            elementClose.innerHTML = options.timerText.replace(/%count%/, '<span class="banner-timer">'+ options.timer +'</span>');
            elementTimer = elementClose.querySelector('.banner-timer') || null;
        } else
            elementClose.innerHTML = options.closeText;
        
        // style
        var elementToCentralize = options.filled ? elementContent : elementBanner;
        this._centralizeStyle(elementToCentralize, options.width, options.height);
        
        if (elementContent) {
            elementContent.style.width = options.width ? options.width +'px' : '';
            elementContent.style.height = options.height ? options.height +'px' : '';
        }
        
        // join
        if (elementContent) elementBanner.appendChild(elementContent);
        elementBanner.appendChild(elementClose);
        if (elementAssist) elementBanner.appendChild(elementAssist);
        
        return {
            'banner' : elementBanner,
            'content' : elementContent,
            'close' : elementClose,
            'assist' : elementAssist,
            'timer' : elementTimer,
            'script' : elementScript,
        };
        
    };
    
    /**
     * @method _setAction
     *
     * @param {Object} Associated array with elements (required keys: banner, timer, close, script)
     * @param {Object} Defined banner options
     */
    this._setAction = function(elements, options) {
        
        var counter = 0;
        
        function timer() {
            if (--counter < 1) {
                clearInterval(interval);
                timerEnded();
            } else {
                elements.timer.innerHTML = counter;
            }
        }
        
        function timerEnded() {
            elements.close.innerHTML = options.closeText;
            elements.close.className = 'banner-close enabled';
            elements.close.addEventListener('click', removeBanner);
        }
        
        function removeBanner() {
            if (elements.banner) elements.banner.parentNode.removeChild(elements.banner);
            if (elements.script) elements.script.parentNode.removeChild(elements.script);
            if (options.callback) options.callback();
        }
        
        if (options.timer) {
            counter = options.timer;
            var interval = setInterval(timer, 1000);
        } else {
            timer();
        }
        
    };
    
    /**
     * @method show
     *
     * @param {Object} options Defined banner options
     */
    this.show = function(options) {
        
        options = this._normalize(options);
        
        if (!options) {
            mp.warning('[MoonplayerBanner] Normalize was be failed');
            return false;
        }
        
        var elements = this._genBannerElements(options);
        
        if (elements.banner) mp.documentLayer.appendChild(elements.banner);
        
        if (elements.script) document.querySelector('head').appendChild(elements.script);
        
        this._setAction(elements, options);
        
    };
    
}