/**
 * @class MoonplayerEvents
 *
 * @param {Object} mp                               Object of Moonplayer Class
 * @property {Object} list                          The object with a events array is sorted by keys
 * @property {Boolean} keepPreventingEventsDiabled  To disable all PREVENT's events
 *
 * @method _bindOnPlayIntent                        To bind ON_PLAY_INTENT event
 * @method _unbindOnPlayIntent                      To stop listening ON_PLAY_INTENT event
 * @method _triggerOnPlayIntent                     Triggering ON_PLAY_INTENT event
 *
 * @method _bindOnLoadMetadata                      To bind ON_LOAD_METADATA event
 * @method _unbindOnLoadMetadata                    To stop listening ON_LOAD_METADATA event
 * @method _triggerOnLoadMetadata                   Triggering ON_LOAD_METADATA event
 *
 * @method _bindOnFirstClickToPlay                  To bind ON_FIRST_CLICK_TO_PLAY event
 * @method _unbindOnFirstClickToPlay                To stop listening ON_FIRST_CLICK_TO_PLAY event
 * @method _triggerOnFirstClickToPlay               Triggering ON_FIRST_CLICK_TO_PLAY event
 *
 * @method _bindOnFirstPlay                         To bind ON_FIRST_PLAY event
 * @method _unbindOnFirstPlay                       To stop listening ON_FIRST_PLAY event
 * @method _triggerOnFirstPlay                      Triggering ON_FIRST_PLAY event
 *
 * @method _bindOnFirstPause                        To bind ON_FIRST_PAUSE event
 * @method _unbindOnFirstPause                      To stop listening ON_FIRST_PAUSE event
 * @method _triggerOnFirstPause                     Triggering ON_FIRST_PAUSE event
 *
 * @method _bindOnPlay                              To bind ON_PLAY event
 * @method _unbindOnPlay                            To stop listening ON_PLAY event
 * @method _triggerOnPlay                           Triggering ON_PLAY event
 *
 * @method _bindOnPause                             To bind ON_PAUSE event
 * @method _unbindOnPause                           To stop listening ON_PAUSE event
 * @method _triggerOnPause                          Triggering ON_PAUSE event
 *
 * @method _bindOnEnded                             To bind ON_ENDED event
 * @method _unbindOnEnded                           To stop listening ON_ENDED event
 * @method _triggerOnEnded                          Triggering ON_ENDED event
 *
 * @method _bindOnMcRendered                        To bind ON_MC_RENDERED event
 * @method _unbindOnMcRendered                      To stop listening ON_MC_RENDERED event
 * @method _triggerOnMcRendered                     Triggering ON_MC_RENDERED event
 *
 * @method _bindOnMcHide                            To bind ON_MC_HIDE event
 * @method _unbindOnMcHide                          To stop listening ON_MC_HIDE event
 * @method _triggerOnMcHide                         Triggering ON_MC_HIDE event
 *
 * @method _bindOnMcShow                            To bind ON_MC_SHOW event
 * @method _unbindOnMcShow                          To stop listening ON_MC_SHOW event
 * @method _triggerOnMcShow                         Triggering ON_MC_SHOW event
 *
 * @method _triggerOnCreated                        Triggering ON_CREATED event (fire on a this.bind method)
 * @method _triggerOnRefresh                        Triggering ON_REFRESH event (fire on a this.refresh method)
 * @method _triggerPreventPlay                      Call each PREVENT_PLAY's functions (stopPropagation)
 *
 * @method bind                                     To bind each events after player was created
 * @method refresh                                  To refresh events after load new source
 */
function MoonplayerEvents(mp) {

	var self = this;

	this.list = {};
    this.keepPreventingEventsDiabled = false;
    
     /**
	 * @method _bindOnPlayIntent
	 */
	this._bindOnPlayIntent = function() {
		mp.player.engine.core.getCurrentPlayback().once(Clappr.Events.PLAYBACK_PLAY_INTENT, self._triggerOnPlayIntent);
	};
    
    /**
     * @method _unbindOnPlayIntent
     */
    this._unbindOnPlayIntent = function() {
        mp.player.engine.core.getCurrentPlayback().off(Clappr.Events.PLAYBACK_PLAY_INTENT, self._triggerOnPlayIntent);
    };
    
    /**
     * @method _triggerOnLoadMetadata
     */
    this._triggerOnPlayIntent = function() {
        for (var i = 0; i < self.list.ON_PLAY_INTENT.length; i++) {
			if (self.list.ON_PLAY_INTENT[i]) self.list.ON_PLAY_INTENT[i](mp);
		}
    };

	/**
	 * @method _bindOnLoadMetadata
	 */
	this._bindOnLoadMetadata = function() {
		mp.player.engine.core.getCurrentContainer().once(Clappr.Events.CONTAINER_LOADEDMETADATA, self._triggerOnLoadMetadata);
	};
    
    /**
     * @method _unbindOnLoadMetadata
     */
    this._unbindOnLoadMetadata = function() {
        mp.player.engine.core.getCurrentContainer().off(Clappr.Events.CONTAINER_LOADEDMETADATA, self._triggerOnLoadMetadata);
    };
    
    /**
     * @method _triggerOnLoadMetadata
     */
    this._triggerOnLoadMetadata = function() {
        for (var i = 0; i < self.list.ON_LOAD_METADATA.length; i++) {
			if (self.list.ON_LOAD_METADATA[i]) self.list.ON_LOAD_METADATA[i](mp);
		}
        // drop PREVENT_PLAY and ON_FIRST_CLICK_TO_PLAY events (fix autoplay)
        self._unbindOnFirstClickToPlay();
    };

	/**
	 * @method _bindOnFirstClickToPlay
	 */
	this._bindOnFirstClickToPlay = function() {
		mp.player.engine.core.el.addEventListener('click', self._triggerOnFirstClickToPlay, true);
	};
    
    /**
	 * @method _unbindOnFirstClickToPlay
	 */
	this._unbindOnFirstClickToPlay = function() {
		mp.player.engine.core.el.removeEventListener('click', self._triggerOnFirstClickToPlay, true);
	};
    
    /**
	 * @method _triggerOnFirstClickToPlay
	 */
	this._triggerOnFirstClickToPlay = function(e) {
		for (var i = 0; i < self.list.ON_FIRST_CLICK_TO_PLAY.length; i++) {
			if (self.list.ON_FIRST_CLICK_TO_PLAY[i]) self.list.ON_FIRST_CLICK_TO_PLAY[i](mp);
		}
        // Call PREVENT_PLAY's
        self._triggerPreventPlay(e);
        // remove listener
		mp.player.engine.core.el.removeEventListener('click', self._triggerOnFirstClickToPlay, true);
	};

	/**
	 * @method _bindOnFirstPlay
	 */
	this._bindOnFirstPlay = function() {
		mp.player.engine.once(Clappr.Events.PLAYER_PLAY, self._triggerOnFirstPlay);
	};
    
    /**
	 * @method _unbindOnFirstPlay
	 */
	this._unbindOnFirstPlay = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PLAY, self._triggerOnFirstPlay);
	};
    
    /**
     * @method _triggerOnFirstPlay
     */
    this._triggerOnFirstPlay = function() {
        for (var i = 0; i < self.list.ON_FIRST_PLAY.length; i++) {
			if (self.list.ON_FIRST_PLAY[i]) self.list.ON_FIRST_PLAY[i](mp);
		}
    };
    
    /**
	 * @method _bindOnFirstPause
	 */
	this._bindOnFirstPause = function() {
		mp.player.engine.once(Clappr.Events.PLAYER_PAUSE, self._triggerOnFirstPause);
	};
    
    /**
	 * @method _unbindOnFirstPause
	 */
	this._unbindOnFirstPause = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PAUSE, self._triggerOnFirstPause);
	};
    
    /**
     * @method _triggerOnFirstPause
     */
    this._triggerOnFirstPause = function() {
        for (var i = 0; i < self.list.ON_FIRST_PAUSE.length; i++) {
			if (self.list.ON_FIRST_PAUSE[i]) self.list.ON_FIRST_PAUSE[i](mp);
		}
    };
    
    /**
	 * @method _bindOnPlay
	 */
    this._bindOnPlay = function() {
        mp.player.engine.on(Clappr.Events.PLAYER_PLAY, self._triggerOnPlay);
    };
    
    /**
	 * @method _unbindOnPlay
	 */
    this._unbindOnPlay = function() {
        mp.player.engine.off(Clappr.Events.PLAYER_PLAY, self._triggerOnPlay);
    };
    
    /**
     * @method _triggerOnPlay
     */
    this._triggerOnPlay = function() {
        for (var i = 0; i < self.list.ON_PLAY.length; i++) {
			if (self.list.ON_PLAY[i]) self.list.ON_PLAY[i](mp);
		}
    };

	/**
	 * @method _bindOnPause
	 */
	this._bindOnPause = function() {
		mp.player.engine.on(Clappr.Events.PLAYER_PAUSE, self._triggerOnPause);
	};
    
    /**
	 * @method _unbindOnPause
	 */
	this._unbindOnPause = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_PAUSE, self._triggerOnPause);
	};
    
    /**
	 * @method _triggerOnPause
	 */
    this._triggerOnPause = function() {
        for (var i = 0; i < self.list.ON_PAUSE.length; i++) {
			if (self.list.ON_PAUSE[i]) self.list.ON_PAUSE[i](mp);
		}
    };

	/**
	 * @method _bindOnEnded
	 */
	this._bindOnEnded = function() {
		mp.player.engine.on(Clappr.Events.PLAYER_ENDED, self._triggerOnEnded);
	};
    
    /**
	 * @method _unbindOnEnded
	 */
	this._unbindOnEnded = function() {
		mp.player.engine.off(Clappr.Events.PLAYER_ENDED, self._triggerOnEnded);
	};
    
    /**
     * @method _triggerOnEnded
     */
    this._triggerOnEnded = function() {
        for (var i = 0; i < self.list.ON_ENDED.length; i++) {
			if (self.list.ON_ENDED[i]) self.list.ON_ENDED[i](mp);
		}
    };
    
    /**
     * @method _bindOnMcRendered
     */
    this._bindOnMcRendered = function() {
        mp.player.engine.listenToOnce(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcRendered);
    };
    
    /**
     * @method _unbindOnMcRendered
     */
    this._unbindOnMcRendered = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcRendered);
    };
    
    /**
     * @method _triggerOnMcRendered
     */
    this._triggerOnMcRendered = function() {
        for (var i = 0; i < self.list.ON_MC_RENDERED.length; i++) {
			if (self.list.ON_MC_RENDERED[i]) self.list.ON_MC_RENDERED[i](mp);
		}
    };
    
    /**
	 * @method _bindOnMcHide
	 */
    this._bindOnMcHide = function() {
        mp.player.engine.listenTo(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_HIDE, self._triggerOnMcHide);
    };
    
    /**
	 * @method _unbindOnMcHide
	 */
    this._unbindOnMcHide = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_HIDE, self._triggerOnMcHide);
    };
    
    /**
     * @method _triggerOnMcHide
     */
    this._triggerOnMcHide = function() {
        for (var i = 0, l = self.list.ON_MC_HIDE.length; i < l; i++) {
			if (self.list.ON_MC_HIDE[i]) self.list.ON_MC_HIDE[i](mp);
		}
    };
    
    /**
	 * @method _bindOnMcShow
	 */
    this._bindOnMcShow = function() {
        mp.player.engine.listenTo(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcShow);
    };
    
    /**
	 * @method _unbindOnMcShow
	 */
    this._unbindOnMcShow = function() {
        mp.player.engine.stopListening(mp.player.engine.core.mediaControl, Clappr.Events.MEDIACONTROL_SHOW, self._triggerOnMcShow);
    };
    
    /**
     * @method _triggerOnMcShow
     */
    this._triggerOnMcShow = function() {
        for (var i = 0, l = self.list.ON_MC_SHOW.length; i < l; i++) {
			if (self.list.ON_MC_SHOW[i]) self.list.ON_MC_SHOW[i](mp);
		}
    };
    
    /**
     * @method _triggerOnCreated
     */
    this._triggerOnCreated = function() {
        for (var i = 0; i < self.list.ON_CREATED.length; i++) {
			if (self.list.ON_CREATED[i]) self.list.ON_CREATED[i](mp);
		}
    };
    
    /**
     * @method _triggerOnRefresh
     */
    this._triggerOnRefresh = function() {
        for (var i = 0; i < self.list.ON_REFRESH.length; i++) {
			if (self.list.ON_REFRESH[i]) self.list.ON_REFRESH[i](mp);
		}
    };
    
    /**
	 * @method _triggerPreventPlay
	 */
	this._triggerPreventPlay = function(e) {
        if (self.list.PREVENT_PLAY[0] && !this.keepPreventingEventsDiabled) {
            e.stopPropagation();
            for (var i = 0; i < self.list.PREVENT_PLAY.length; i++) {
            	if (self.list.PREVENT_PLAY[i]) self.list.PREVENT_PLAY[i](mp);
            }
        }
    };

	/**
	 * @method bind
	 */
	this.bind = function() {
        this._bindOnPlayIntent();
		this._bindOnLoadMetadata();
		this._bindOnFirstClickToPlay();
		this._bindOnFirstPlay();
        this._bindOnFirstPause();
        this._bindOnPlay();
		this._bindOnPause();
		this._bindOnEnded();
        this._bindOnMcRendered();
        this._bindOnMcHide();
        this._bindOnMcShow();
        
        this._triggerOnCreated();
	};
    
    /**
     * @method refresh
     */
    this.refresh = function() {
        this._unbindOnPlayIntent();
        this._unbindOnLoadMetadata();
        this._unbindOnFirstClickToPlay();
        this._unbindOnFirstPlay();
        this._unbindOnFirstPause();
        this._unbindOnPlay();
        this._unbindOnEnded();
        this._unbindOnMcRendered();
        this._unbindOnMcHide();
        this._unbindOnMcShow();
        
        this._bindOnPlayIntent();
        this._bindOnLoadMetadata();
        this._bindOnFirstClickToPlay();
        this._bindOnFirstPlay();
        this._bindOnFirstPause();
        this._bindOnPlay();
        this._bindOnEnded();
        this._bindOnMcRendered();
        this._bindOnMcHide();
        this._bindOnMcShow();
        
        this._triggerOnRefresh();
    };
    
	this.list.ON_CREATED = [];
    this.list.ON_REFRESH = [];
    this.list.ON_PLAY_INTENT = [];
	this.list.ON_LOAD_METADATA = [];
	this.list.ON_FIRST_CLICK_TO_PLAY = [];
	this.list.ON_FIRST_PLAY = [];
    this.list.ON_FIRST_PAUSE = [];
    this.list.ON_PLAY = [];
	this.list.ON_PAUSE = [];
	this.list.ON_ENDED = [];
    this.list.ON_MC_RENDERED = [];
    this.list.ON_MC_HIDE = [];
    this.list.ON_MC_SHOW = [];
	this.list.PREVENT_PLAY = [];
}