/**
 * @class MoonplayerPlayer
 *
 * @property {Object} engine            Object of Clappr.Player Class
 * @property {Object} elementContainer  DOMElement #player .container
 * @method create                       Initialize Clappr with source from MoonplayerData Class
 * @method updateSource                 To load new source from MoonplayerData Class
 * @method _isFlash                     Return true if engine uses flash
 * @method buffering                    To try to start buffering
 * @method showMessage                  To show player message
 * @method hideMessage                  To hide player message
 */
function MoonplayerPlayer(mp) {

    var self = this;

	this.engine = null;
	this.elementContainer = null;

	/**
	 * @method create
	 */
	this.create = function() {

		this.engine = new Clappr.Player({
			width: '100%',
			height: '100%',
			parent: mp.documentPlayer,
			source: mp.data.json.manifest,
			baseUrl: mp.options.assetsPath,
			autoplay: false,
			preload: 'metadata',
			poster: mp.options.posterUrl ? mp.options.posterUrl : mp.data.json.poster ? mp.data.json.poster : undefined,
			plugins: {
				core: [LevelSelector, ClapprSubtitle],
				playback: mp.options.flashEnabled ? [Clappr.FlasHLS] : []
			},
			autoSeekFromUrl: false,
			exitFullscreenOnEnd: false,
			levelSelectorConfig: {
				labels: {
					3: '1080p',
					2: '720p',
					1: '480p',
					0: '360p'
				}
			},
			playbackNotSupportedMessage: mp.options.notSupportedMsg,
            subtitle: ''
		});

		this.elementContainer = this.engine.core.el.querySelector('.container');

	};
    
    /**
     * @method updateSource
     */
    this.updateSource = function() {
        if (self.engine) self.engine.load({source: mp.data.json.manifest, mimeType: 'application/x-mpegURL'});
    };
    
    /**
     * @method _isFlash
     *
     * @returns {Boolean|Null} Return true if engine uses flash
     */
    this._isFlash = function() {
        return self.engine ? !!self.engine.getPlugin('flashls') : null;
    };
    
    /**
     * @method buffering
     */
    this.buffering = function() {
        if (self.engine && !self._isFlash()) {
            self.engine.play();
            self.engine.pause();
        } else if (self.engine) {
            self.engine.play();
            self.engine.core.getCurrentContainer().once(
                Clappr.Events.CONTAINER_LOADEDMETADATA,
                function() {
                    self.engine.pause();
                }
            );
        }
    };
    
    /**
     * @method showMessage
     *
     * @param {String} message      Message to show
     * @param {Boolean} autohide    Hide message automatically
     */
    this.showMessage = function(message, autohide) {
        var seekTimePlugin = mp.player.engine.core.getPlugin('seek_time');
        if (!seekTimePlugin.hoverPosition) seekTimePlugin.hoverPosition = 0;
        seekTimePlugin.hoveringOverSeekBar = true;
        seekTimePlugin.update();
        seekTimePlugin.$seekTimeEl[0].innerHTML = message;
        if (autohide) this.hideMessage(1000);
    };
    
    /**
     * @method hideMessage
     *
     * @param {Number} time     Time (ms) to hide message
     */
    this.hideMessage = function(time) {
        var seekTimePlugin = mp.player.engine.core.getPlugin('seek_time');
        setTimeout(function() {
            seekTimePlugin.hoveringOverSeekBar = false;
            seekTimePlugin.update();
        }, time || 1);
    };

}