/**
 * @class MoonplayerPlaylist
 *
 * @param {Object} mp Object of Moonplayer Class
 * @method _getCurrentIndexes Returns object contain indexes by season, spisode and release keys
 * @method getCurrentNames Returns names of current season, episode and release
 * @method getNextNames Returns names of next season and episode with current release
 * @method getUrlByNames Returns url string by season and episode name
 */
function MoonplayerPlaylist(mp) {

    /**
     * @method _getCurrentIndexes
     *
     * @returns {Object} Object contain indexes by season, spisode and release keys
     */
    this._getCurrentIndexes = function() {
        var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
			indexSeason = null,
			indexEpisode = null,
			indexRelease = null;
        for (var i = 0, l = seasons.length; i < l; i++) {
			if (seasons[i].selected) indexSeason = i;
		}
		for (var i = 0, l = episodes.length; i < l; i++) {
			if (episodes[i].selected) indexEpisode = i;
		}
		for (var i = 0, l = releases.length; i < l; i++) {
			if (releases[i].selected) indexRelease = i;
		}
        return {
			season: indexSeason,
			episode: indexEpisode,
            release: indexRelease
		};
    };

    /**
	 * @method getCurrentNames
	 *
	 * @returns {Object} Object contain names by season, episode and release keys
	 */
	this.getCurrentNames = function() {
		var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
            indexes = this._getCurrentIndexes();
		return {
			season: indexes.season !== null ? seasons[indexes.season].name : null,
			episode: indexes.episode !== null ? episodes[indexes.episode].name : null,
            release: indexes.release !== null ? releases[indexes.release].name : null
		};
	};
    
    /**
     * @method getNextNames
     *
     * @returns {Object} Object contain names by season, episode and release keys
     */
    this.getNextNames = function() {
        var seasons = mp.data.json.playlist.seasons,
			episodes = mp.data.json.playlist.episodes,
            releases = mp.data.json.playlist.releases,
            indexes = this._getCurrentIndexes(),
            namesNext = Object.create(null);
        
        namesNext.season =
        namesNext.episode =
        namesNext.release = null;
        
        // current release
        if (indexes.release !== null && indexes.release in releases)
            namesNext.release = releases[indexes.release].name;
        
        // season is not exists
        if (indexes.season === null) return namesNext;
        
        // episode is exist and not is latest
        if (indexes.episode !== null && indexes.episode +1 in episodes) {
            namesNext.season = seasons[indexes.season].name;
            namesNext.episode = episodes[indexes.episode +1].name;
            return namesNext;
        }
        
        // episode is latest
        namesNext.season = indexes.season +1 in seasons ? seasons[indexes.season +1].name : seasons[0].name;
        return namesNext;
    };

	/**
	 * @method getUrlByNames
	 *
	 * @param {String} Name of season
	 * @param {String} Name of episode
	 * @returns {String|null} Url
	 */
	this.getUrlByNames = function(nameSeason, nameEpisode) {
		var url = null,
			seasons = mp.data.json.playlist.seasons;
		
        if (nameSeason === null) return null;
		
        for (var i = 0, l = seasons.length; i < l; i++) {

			if (seasons[i].name === nameSeason) {

                url = seasons[i].url;
                break;
            }
		}
        
        if (url && nameEpisode !== null)
            url = url.replace(/episode=(.*?)[^&]/, 'episode='+ nameEpisode);
        else if (url)
            url = url.replace(/episode=(.*?)[^&]/, 'episode=1');
        
		return url;
	};

}