/**
 * @class MoonplayerPlaybar
 *
 * @param {Object} mp Object of Moonplayer Class
 * @property {Object} el DOMElement #playbar
 * @method _genButtonElement Generate playbar button from array list
 * @method flush Create or update playbar elements
 */
function MoonplayerPlaybar(mp) {

	var self = this;

	this.el = null;

	/**
	 * @method _genButtonElement
	 *
	 * @param {Array} listArray Important value! Array of entity with keys: name, url, selected
	 * @param {String} arrayName Important value! The class of button element
	 * @param {String} buttonLabel Important value! The label of button element (pattern %name%)
	 * @param {String} itemLabel The label of list item (pattern %name%)
	 * @param {String} noLabel Alternative name for a not existing name value of list item (pattern %index%)
	 * @param {Function} onClick The Callback function on click event for each anchor elements
	 * @param {Number} nameLength Maximum name characters for split by space
	 */
	this._genButtonElement = function(listArray, arrayName, buttonLabel, itemLabel, noLabel, onClick, nameLength) {
		var documentButton = document.createElement('div'),
			documentLabel = document.createElement('span'),
			documentList = document.createElement('ul');

		if (!listArray.length) documentButton.style.display = 'none';
		documentButton.className = 'playbar-button playbar-button-' + arrayName;
		documentLabel.innerHTML = buttonLabel.replace(/\s?%name%\s?/g, '');

		for (var i = 0, l = listArray.length; i < l; i++) {
			var documentAnchor = document.createElement('a');
			var documentItem = document.createElement('li');

			// anchor event
			if (onClick) documentAnchor.addEventListener('click', onClick);

			// anchor name
			documentAnchor.innerHTML = itemLabel && listArray[i].name && /%name%/.test(itemLabel) ?
				itemLabel.replace(/%name%/g, listArray[i].name) : listArray[i].name ?
				listArray[i].name : /%index%/.test(noLabel) ?
				noLabel.replace(/%index%/g, i + 1) : noLabel || 'undefined';

            // anchor name length
            if (nameLength && documentAnchor.innerHTML[nameLength])
                documentAnchor.innerHTML = documentAnchor.innerHTML.split(' ')[0];

			// url
			if (listArray[i].url) documentAnchor.setAttribute('href', listArray[i].url);

			// selected
			if (listArray[i].selected) {
				documentItem.className = 'selected';
				documentLabel.innerHTML = listArray[i].name && /%name%/.test(buttonLabel) ?
					buttonLabel.replace(/%name%/g, listArray[i].name) : buttonLabel;
			}

			documentItem.appendChild(documentAnchor);
			documentList.appendChild(documentItem);
		}

		documentButton.appendChild(documentLabel);
		documentButton.appendChild(documentList);

		return documentButton;
	};

	/**
	 * @method flush
	 */
	this.flush = function() {

		if (!this.el) {
			this.el = document.createElement('div');
			this.el.setAttribute('id', 'playbar');
			mp.documentPlayer.appendChild(this.el);
			// events: show or hide with mediacontrol
			mp.events.list.ON_MC_HIDE.push(function() {
				self.el.className = 'hide';
			});
			mp.events.list.ON_MC_SHOW.push(function() {
				self.el.className = '';
			});
            // events: force to show after ended
			mp.events.list.ON_ENDED.push(function() {
				self.el.className = '';
                mp.player.engine.core.mediaControl.keepVisible = true;
			});
            // events: force to show mediacontrol on hover on playbar
			this.el.addEventListener('mouseenter', function() {
				mp.player.engine.core.mediaControl.show();
				mp.player.engine.core.mediaControl.keepVisible = true;
			});
			this.el.addEventListener('mouseleave', function() {
				if (mp.player.engine.isPlaying()) mp.player.engine.core.mediaControl.keepVisible = false;
			});
		}

		this.el.innerHTML = '';
        this.el.className = '';

        var arrowHTML = '<svg xmlns="http://www.w3.org/2000/svg" height="9" width="11" viewBox="0 0 11 9"><g transform="matrix(.027363 0 0 .024322 2 .50211)"><path d="m286.94 70.277c-3.614-3.617-7.898-5.424-12.848-5.424h-255.81c-4.952 0-9.233 1.807-12.85 5.424-3.617 3.621-5.424 7.902-5.424 12.851 0 4.948 1.807 9.229 5.424 12.847l127.9 127.9c3.621 3.617 7.902 5.428 12.85 5.428s9.233-1.811 12.847-5.428l127.91-127.91c3.613-3.617 5.427-7.898 5.427-12.847 0-4.948-1.814-9.229-5.427-12.85z"/></g></svg>';

        // enconde cyrillic characters
        var seasonCyr  = decodeURIComponent('%D1%81%D0%B5%D0%B7%D0%BE%D0%BD');
        var episodeCyr = decodeURIComponent('%D1%81%D0%B5%D1%80%D0%B8%D1%8F');
        var releaseCyr = decodeURIComponent('%D0%BF%D0%B5%D1%80%D0%B5%D0%B2%D0%BE%D0%B4');
        var emptyCyr   = decodeURIComponent('%D0%B1%D0%B5%D0%B7 %D0%B8%D0%BC%D0%B5%D0%BD%D0%B8');

		this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.seasons,
			'seasons',
            seasonCyr +' %name% '+ arrowHTML,
            seasonCyr +' %name%',
            emptyCyr,
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            }
		));

		this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.episodes,
			'episodes',
            episodeCyr +' %name% '+ arrowHTML,
            episodeCyr +' %name%',
            emptyCyr,
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            }
		));

		if (mp.options.releasesEnabled) this.el.appendChild(this._genButtonElement(
			mp.data.json.playlist.releases,
			'releases',
            releaseCyr +' '+ arrowHTML,
            '%name%',
            releaseCyr +' %index%',
            function(e) {
                var link = this.getAttribute('href');
                if (link) {
                    e.stopPropagation();
                    e.preventDefault();
                    mp.load(link);
                }
            },
            14
		));

        // enconde cyrillic characters
        var playerCyr = decodeURIComponent('%D0%9F%D0%BB%D0%B5%D0%B5%D1%80');
        var linkCyr   = decodeURIComponent('%D0%A1%D1%81%D1%8B%D0%BB%D0%BA%D0%B0');
        var codeCyr   = decodeURIComponent('%D0%BA%D0%BE%D0%B4 %D0%B2%D0%B8%D0%B4%D0%B5%D0%BE');

		if (mp.options.playerCodeEnabled) this.el.appendChild(this._genButtonElement(
			[{
				name: playerCyr +':<br><textarea><iframe src="http://moonwalk.cc' + location.pathname.split('?')[0].replace(mp.options.scriptPath, '') + '" width="610" height="370" frameborder="0" allowfullscreen></iframe></textarea>'
			}, {
				name: linkCyr +':<br><textarea>http://moonwalk.cc' + location.pathname.split('?')[0].replace(mp.options.scriptPath, '') + '</textarea>'
			}],
			'code',
            codeCyr
		));

	};

}