/**
 * @class MoonplayerRollback
 *
 * @param {Object} mp Objec of Moonplayer Class
 * @property {Object} el DOMElement .rollback-bar
 * @method _getNamesFromCookie Return object contain names by season and episode key from cookie
 * @method setCurrentCookie To set 'mp_rollback' cookie by the current season and episode names
 * @method show To create and show a Rollback element
 * @method hide To remove a Rollback element
 */
function MoonplayerRollback(mp) {

    this.el = null;
    
    /**
     * @method _getNamesFromCookie
     *
     * @returns {Object|null} Return object contain names by season and episode key
     */
    this._getNamesFromCookie = function() {
        if (document.cookie.indexOf('mp_rollback=') < 0) return null;
        var names = /mp_rollback=([^;\s]*)/.exec(document.cookie)[1].split('+');
        if (!names[0] || !names[1]) return null;
        return {season: names[0], episode: names[1]};
    };
    
    /**
     * @method setCurrentCookie
     */
    this.setCurrentCookie = function() {
        var names = mp.playlist.getCurrentNames(),
            expires = new Date();
        expires.setDate(expires.getDate() + 14);
        if (!names.season || !names.episode) return false;
        document.cookie = 'mp_rollback=' + names.season + '+' + names.episode + '; expires=' + expires.toUTCString();
    };
    
    /**
     * @method show
     */
    this.show = function() {
        // get names
        var namesCookie = this._getNamesFromCookie(),
            namesCurrent = mp.playlist.getCurrentNames();
        // return if names not defined
        if (!namesCookie || !namesCurrent.season || !namesCurrent.episode) return false;
        // return if names is equal
        if (namesCookie.season === namesCurrent.season && namesCookie.episode === namesCurrent.episode) return false;
        // get url by season and episode from cookie
        var url = mp.playlist.getUrlByNames(namesCookie.season, namesCookie.episode);
        // return if url is not defined
        if (!url) return false;
        // create rollback element if not exists
        if (!this.el) {
            this.el = document.createElement('div');
            this.el.className = 'rollback-bar';
        }
        this.el.innerHTML =
            decodeURIComponent('%D0%92%D1%8B %D0%BE%D1%81%D1%82%D0%B0%D0%BD%D0%BE%D0%B2%D0%B8%D0%BB%D0%B8%D1%81%D1%8C %D0%BD%D0%B0 ')
            + namesCookie.episode
            + decodeURIComponent(' %D1%81%D0%B5%D1%80%D0%B8%D0%B8 ')
            + namesCookie.season
            + decodeURIComponent(' %D1%81%D0%B5%D0%B7%D0%BE%D0%BD%D0%B0 ')
            + '<a href="' + url + '">'
            + decodeURIComponent('%D0%BF%D0%B5%D1%80%D0%B5%D0%B9%D1%82%D0%B8%C2%BB')
            + '</a>';

        mp.documentLayer.appendChild(this.el);
    };
    
    /**
     * @method hide
     */
    this.hide = function() {
        if (this.el && this.el.parentNode) this.el.parentNode.removeChild(this.el);
    };
    
}