/**
 * @class Moonplayer
 *
 * @property {Object} options           Object of Settings Data
 * @property {Object} documentLayer     DOMElement #layer
 * @property {Object} documentPlayer    DOMElement #player
 * @property {Object} presets           Object of MoonplayerPresets Class
 * @property {Object} data              Object of MoonplayerData Class
 * @property {Object} spinner           Object of MoonplayerData Class
 * @property {Object} player            Object of MoonplayerPlayer Class
 * @property {Object} events            Object of MoonplayerEvents Class
 * @property {Object} playbar           Object of MoonplayerPlaybar Class
 * @property {Object} playlist          Object of MoonplayerPlaylist Class
 * @property {Object} rollback          Object of MoonplayerRollback Class
 * @property {Object} autoplay          Object of MoonplayerAutoplay Class
 * @property {Object} banner            Object of MoonplayerBanner Class
 * @property {Object} execute           Object of MoonplayerExecute Calss
 * @method init                         Initialize Moonplayer
 * @method load                         To load source by iframe url (ajax)
 * @method error                        Thrown exception
 * @method warning                      Thrown warning
 * @method isMobile                     Returns true if mobile browser was detected
 */
function Moonplayer() {

	var self = this;

	this.options = {};

	this.documentLayer = document.getElementById('layer');
	this.documentPlayer = document.getElementById('player');

	this.presets = new MoonplayerPresets(this);
	this.data = new MoonplayerData(this);
	this.spinner = new MoonplayerSpinner(this);
	this.player = new MoonplayerPlayer(this);
	this.events = new MoonplayerEvents(this);
	this.playbar = new MoonplayerPlaybar(this);
    this.playlist = new MoonplayerPlaylist(this);
	this.rollback = new MoonplayerRollback(this);
	this.autoplay = new MoonplayerAutoplay(this);
    this.banner = new MoonplayerBanner(this);
    this.execute = new MoonplayerExecute(this);

	/**
	 * @method init
	 */
	this.init = function() {
		// presets
		this.presets.fixFirefoxComutedStyle();
		this.presets.rewriteFullscreen();
		this.presets.setMediaControlTemplate();
        this.presets.showMediaControlOnPause();
        this.presets.preventClickAfterPlayIntentOnFlash();
		// show spinner
		this.spinner.show();
		// query
		this.data.load(this.options.parserData, function(/*json*/) {

            /*
            self.events.list.ON_CREATED.push(function() { console.log('ON_CREATED'); });
			self.events.list.ON_REFRESH.push(function() { console.log('ON_REFRESH'); });
            self.events.list.ON_PLAY_INTENT.push(function() { console.log('ON_PLAY_INTENT'); });
			self.events.list.ON_LOAD_METADATA.push(function() { console.log('ON_LOAD_METADATA'); });
			self.events.list.ON_FIRST_CLICK_TO_PLAY.push(function() { console.log('ON_FIRST_CLICK_TO_PLAY'); });
			self.events.list.ON_FIRST_PLAY.push(function() { console.log('ON_FIRST_PLAY'); });
            self.events.list.ON_PLAY.push(function() { console.log('ON_PLAY'); });
			self.events.list.ON_PAUSE.push(function() { console.log('ON_PAUSE'); });
            self.events.list.ON_ENDED.push(function() { console.log('ON_ENDED'); });
            self.events.list.ON_MC_RENDERED.push(function() { console.log('ON_MC_RENDERED'); });
            self.events.list.ON_MC_HIDE.push(function() { console.log('ON_MC_HIDE'); });
			self.events.list.ON_MC_SHOW.push(function() { console.log('ON_MC_SHOW'); });
			self.events.list.PREVENT_PLAY.push(function() { console.log('PREVENT_PLAY'); self.player.engine.play(); });
            */
            
			// rollback
			if (self.options.rollbackEnabled) {
				self.rollback.show();
				self.events.list.ON_FIRST_PLAY.push(function() {
					self.rollback.hide();
					self.rollback.setCurrentCookie();
				});
			}

			// autoplay
            if (self.options.autoplayEnabled && !self.isMobile()) {
                self.events.list.ON_MC_RENDERED.push(function() {
                    self.autoplay.flush();
                });
                self.events.list.ON_ENDED.push(function() {
                    self.autoplay.next();
                });
            }
            
            // force to hide spinner
            self.events.list.ON_LOAD_METADATA.push(function() {
                self.spinner.hide();
            });
            
			self.player.create();
			self.events.bind();
			self.playbar.flush();
			self.spinner.hide();
		});
	};

	/**
	 * @method load
	 */
	this.load = function(url, autoplay) {
		// show spinner
		this.spinner.show();
        // stop playing
        if (self.player.engine.isPlaying()) self.player.engine.pause();
		// query
		this.data.load(this.data.urlToData(url), function(/*json*/) {
			self.player.updateSource();
			self.events.refresh();
			self.playbar.flush();
            if (autoplay) self.player.engine.play();
            else self.spinner.hide()
		});
	};

	/**
	 * @method error
	 */
	this.error = function(msg) {
		location.href = this.options.realUrl;
		throw msg || '[Moonplayer] unknown error';
	};
    
    /**
     * @method warning
     */
    this.warning = function(msg) {
        console.warn(msg || '[Moonplayer] unknown warning');
    };
    
    /**
     * @method isMobile
     *
     * @returns {Boolean} Returns true if mobile browser was detected
     */
    this.isMobile = function() {
        return /android|blackberry|iphone|ipad|ipod|opera mini|iemobile/i.test(navigator.userAgent);
    };
}