<?php

namespace Moonlight\Component\Client;

use GuzzleHttp\Client as BaseClient;
use GuzzleHttp\Exception\TransferException;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Webmozart\Assert\Assert;

class Client extends BaseClient implements LoggerAwareInterface
{
    use LoggerAwareTrait;

    protected $retryLimit = 0;
    protected $retryDelay = 250000;
    protected $proxyList  = [];

    public function __construct(array $config = [])
    {
        if (isset($config['proxy']) && is_string($config['proxy']))
            $config['proxy'] = $this->normalizeProxy($config['proxy']);

        parent::__construct($config);
    }

    public function normalizeProxy($proxy, $protocol = 'http')
    {
        Assert::stringNotEmpty(
            $proxy,
            'Proxy server of client requests must be not empty string, %s given.'
        );

        Assert::stringNotEmpty(
            $protocol,
            'Default protocol of client requests must be not empty string, %s given.'
        );

        // replace @ (at) symbol in login and password to "%40"
        $string = preg_replace('/([@])(?=.+@)/', '%40', $proxy);

        // prepend not existed protocol (http as defaults)
        $string = preg_replace('/^(?!.{2,5}:\/\/)/', $protocol .'://', $string);

        // validate
        if (preg_match('/^([^:]{2,5}:\/\/)?([^:\s@\/]+:[^:\s@\/]+@)?[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}:[0-9]{1,6}$/', $string))
            return $string;

        throw new \ParseError(sprintf('Invalid proxy syntax, %s given.', $proxy));
    }

    protected function headersToString(array $headers)
    {
        $string = '';

        foreach ($headers as $name => $value) {

            $value = is_array($value)
                ? implode(', ', $value)
                : $value;

            $string .= $name .': '. $value .' ';
        }

        return trim($string);
    }

    public function prepareHeaders(array $headers)
    {
        $defaults = $this->getConfig('headers') ?: [];

        $headers = array_merge($defaults, $headers);

        $headersKeys = array_keys($headers);

        $headersValues = array_values($headers);

        $headersKeysLowercase = array_map('strtolower', $headersKeys);

        $headersKeysLowercaseUnique = array_unique($headersKeysLowercase);

        $headersUnique = [];

        foreach ($headersKeysLowercaseUnique as $key => $value) {

            $headersUnique[ $headersKeys[$key] ] = $headersValues[$key];
        }

        return $headersUnique;
    }

    public function setRetryLimit($retryLimit)
    {
        Assert::integer(
            $retryLimit,
            'Retry limit of client requests must be an integer, %s given.'
        );

        Assert::range(
            $retryLimit, 0, 50,
            'Retry limit of client request must be within a range 0-50, %s given.'
        );

        $this->retryLimit = $retryLimit;
    }

    public function setRetryDelay($retryDelay)
    {
        Assert::integer(
            $retryDelay,
            'Delay retry of client requests must be an integer, %s given.'
        );

        Assert::range(
            $retryDelay, 0, 10000000,
            'Delay retry of client request must be within a range 0-10000000 (max 10sec), %s given.'
        );

        $this->retryDelay = $retryDelay;
    }

    public function setProxyList(
        array $proxyList = null,
        $protocol  = 'http',
        $shuffle   = false,
        $listLimit = 0
    ) {
        Assert::nullOrIsArray(
            $proxyList,
            'Proxy list of client requests must be an array or null, %s given.'
        );

        Assert::stringNotEmpty(
            $protocol,
            'Protocol of client requests via proxy must be not empty string, %s given.'
        );

        Assert::integer(
            $listLimit,
            'Limit of the proxy list for client requests must be an integer, %s given.'
        );

        Assert::greaterThanEq(
            $listLimit, 0,
            'Limit of the proxy list for client requests must be equal to 0 or greater, %s given.'
        );

        $proxyList = $proxyList ?: [];

        $list = [];

        foreach ($proxyList as $item) {

            $list[] = $this->normalizeProxy($item, $protocol);
        }

        if ($shuffle) shuffle($list);

        if ($listLimit && count($list) > $listLimit)
            $list = array_slice($list, 0, $listLimit);

        $this->proxyList = $list;
    }

    public function sendRequest($method, $uri = '', array $options = [])
    {
        // define proxy
        $proxy = isset($options['proxy'])
            ? $options['proxy']
            : $this->getConfig('proxy');

        // define proxy from proxy list by index if integer
        if (is_int($proxy) && isset($this->proxyList[$proxy]))
            $proxy = $this->proxyList[$proxy];

        if (is_int($proxy))
            throw new \InvalidArgumentException(sprintf(
                'Client required a non-existent index %s of proxy list.',
                $proxy
            ));

        $options['proxy']   = $proxy;
        $options['headers'] = $this->prepareHeaders(
            isset($options['headers'])
                ? $options['headers']
                : []
        );

        $response = null;

        try {

            $response = parent::request($method, $uri, $options);

            if (preg_match('/Blocked\scontent/', $response->getBody()))
                throw new TransferException('Blocked content', 404);

        } catch (TransferException $exception) {}

        if ($this->logger)
            $this->logger->debug(sprintf(
                'Client %s %d%s%s - %s %s',
                $method,
                isset($exception)
                    ? $exception->getCode()
                    : $response->getStatusCode(),
                isset($exception) ? ' (failed)' : '',
                $proxy ? ' via proxy '. $proxy : '',
                $uri,
                $this->headersToString($options['headers'])
            ));

        if (isset($exception)) throw $exception;

        return $response;
    }

    public function request($method, $uri = '', array $options = [])
    {
        try {

            return $this->sendRequest($method, $uri, $options);

        } catch (TransferException $e) {

            if ($this->logger)
                $this->logger->debug(sprintf(
                    'Client configured on %d retry requests (delay %f sec).',
                    $this->retryLimit,
                    $this->retryDelay / 1000000
                ));

            if (!$this->retryLimit) throw $e;

            if ($this->logger)
                $this->logger->debug(sprintf(
                    'Client found %s proxy servers for retry requests.',
                    count($this->proxyList)
                ));

            $counter = 0;

            if ($this->proxyList) {

                foreach ($this->proxyList as $proxy) {

                    if (++$counter > $this->retryLimit) break;

                    try {

                        $response = $this->sendRequest($method, $uri, array_merge($options, [
                            'proxy' => $proxy
                        ]));

                        $this->proxyList = [$proxy];

                        if ($this->logger)
                            $this->logger->debug(sprintf(
                                'Client proxy server %s saved as main.',
                                $this->proxyList[0]
                            ));

                        return $response;

                    } catch (TransferException $exception) {}
                }
            }

            while ($counter < $this->retryLimit) {

                ++$counter;

                try {

                    return $this->sendRequest($method, $uri, $options);

                } catch (TransferException $exception) {}

                usleep($this->retryDelay);
            }

            throw $e;
        }
    }
}