<?php

namespace Moonlight\Component\Configuration;

use Mikeevstropov\ParameterBag\ParameterBag;
use Webmozart\Assert\Assert;

class Configuration extends ParameterBag
{
    protected $output;
    protected $rootKey;
    protected $annotations;

    /**
     * Configuration constructor.
     *
     * @param string $file    Non-modifiable configuration file
     * @param string $output  Output file for the dump of modified parameters
     * @param string $rootKey Root key for multi-dimensional configuration
     */
    public function __construct(
        $file,
        $output,
        $rootKey = null
    ) {
        Assert::stringNotEmpty(
            $file,
            'Path to the configuration file must be a non-empty string, %s given.'
        );

        Assert::stringNotEmpty(
            $output,
            'Path to the output file of the modified parameters must be a non-empty string, %s given.'
        );

        Assert::nullOrStringNotEmpty(
            $rootKey,
            'Root key for the multi-dimensional configuration must be a non-empty string, %s given.'
        );

        $file = realpath($file) ?: $file;

        Assert::file(
            $file,
            'The configuration file is not a file, %s given.'
        );

        $contents = file_get_contents($file);

        $parameters = Yaml::parse($contents);

        Assert::isArray(
            $parameters,
            "Unable to parse YAML file \"$file\"."
        );

        $output = realpath($output) ?: $output;

        $this->output = $output;

        Assert::false(
            is_dir($output),
            'The output file of the modified parameters is not a file, %s given.'
        );

        Assert::true(
            @touch($output),
            "Unable to write the output file of the modified parameters to \"$output\"."
        );

        $modified = Yaml::parse(file_get_contents($output)) ?: [];

        if ($rootKey) {

            Assert::keyExists(
                $parameters,
                $rootKey,
                'Key %2$s is not exists in the parameters.'
            );

            $parameters = $parameters[$rootKey];
            $modified   = isset($modified[$rootKey])
                ? $modified[$rootKey]
                : [];

            $this->rootKey = $rootKey;
        }

        parent::__construct(array_merge($parameters, $modified));

        $annotations = Yaml::parseAnnotations($contents);

        Assert::isArray(
            $annotations,
            "Unable to parse YAML annotations from file \"$file\"."
        );

        $this->annotations = $annotations;
    }

    /**
     * Dynamic getter for the configuration annotations.
     * Call getInfo('my_var') to get annotation "my_var"
     * of type "info".
     *
     * @param  string     $methodName
     * @param  array      $arguments
     * @return null|mixed
     */
    public function __call($methodName, $arguments)
    {
        if (
            strpos($methodName, 'get') === 0
            && isset($methodName[4])
            && isset($arguments[0])
        ) {
            Assert::stringNotEmpty(
                $arguments[0],
                "Annotation name must be a non-empty string, %s given."
            );

            $type = strtolower(substr($methodName, 3));

            $types = array_change_key_case(
                $this->annotations,
                CASE_LOWER
            );

            $default = isset($arguments[1])
                ? $arguments[1]
                : null;

            if (!key_exists($type, $types)) return $default;

            $annotations = $types[$type];

            $name = $arguments[0];

            if (!key_exists($name, $annotations)) return $default;

            return $annotations[$name];
        }

        throw new \BadMethodCallException('Call to undefined method '. __CLASS__ .'::'. $methodName .'()');
    }

    public function set($name, $value)
    {
        parent::set($name, $value);

        $parameters = $this->all();

        if ($this->rootKey)
            $parameters = [$this->rootKey => $parameters];

        $contents = Yaml::dump($parameters);

        file_put_contents($this->output, $contents, LOCK_EX);
    }
}