<?php

namespace Moonlight\Component\Templating;

use Symfony\Component\Yaml\Yaml;
use Webmozart\Assert\Assert;

class Templating extends \Twig_Environment
{
    protected $path;
    protected $viewDir;
    protected $tempDir;
    protected $controllerName;

    /**
     * Templating constructor.
     *
     * @param string $viewDir        Path to the view directory
     * @param string $tempDir        Path to the temp directory
     * @param string $controllerName Name of the controller
     * @param array  $options        Templating options
     */
    public function __construct(
        $viewDir,
        $tempDir,
        $controllerName,
        array $options = []
    ) {
        Assert::stringNotEmpty(
            $viewDir,
            'Templating expects the path to the view directory must be a non-empty string, %s given.'
        );

        Assert::stringNotEmpty(
            $tempDir,
            'Templating expects the path to the temp directory must be a non-empty string, %s given.'
        );

        Assert::stringNotEmpty(
            $controllerName,
            'Templating expects the controller name must be a non-empty string, %s given.'
        );

        Assert::directory(
            $tempDir,
            'Templating expects the path to the temp directory must be a directory, %s given.'
        );

        $this->tempDir = realpath($tempDir);

        Assert::directory(
            $viewDir,
            'The view directory of template is not a directory, %s given.'
        );

        $viewDir = realpath($viewDir);

        $this->viewDir = $viewDir;

        $path = $viewDir .'/'. $controllerName;

        Assert::directory(
            $path,
            'Path to the template directory is not a directory, %s given.'
        );

        $this->path = realpath($path);

        $loader = new \Twig_Loader_Filesystem($path);

        parent::__construct($loader, $options);

        $this->addFilter(new \Twig_SimpleFilter('type', 'gettype'));
        $this->addFilter(new \Twig_SimpleFilter('field_type', [$this, 'getFieldType']));
        $this->addFunction(new \Twig_SimpleFunction('set_root', [$this, 'setRoot']));
    }

    public function getFieldType($value)
    {
        switch (gettype($value)) {
            case 'boolean':
                return 'switch';
                break;
            case 'integer':
                return 'input';
                break;
            case 'double':
                return 'input';
                break;
            case 'string':
                return 'input';
                break;
            case 'array':
                return 'list';
                break;
            case 'object':
                return null;
                break;
            case 'resource':
                return null;
                break;
            case 'NULL':
                return null;
                break;
            case 'unknown type':
                return null;
                break;
        }

        return null;
    }

    public function setRoot($path, $root)
    {
        if (
            strpos($path, '//') === false
            && strpos($path, '/') !== 0
        ) {

            return preg_replace('/\/$/', '', $root) .'/'. $path;
        }

        return $path;
    }

    public function getViewDir()
    {
        return $this->viewDir;
    }

    public function loadContext($file)
    {
        Assert::stringNotEmpty(
            $file,
            'File name of the template context must be a non-empty string, %s given.'
        );

        $path = $this->path .'/'. $file;

        Assert::file(
            $path,
            'Context file of the template is not a file, %s given.'
        );

        Assert::readable(
            $path,
            'Context file of the template is not readable, %s given.'
        );

        $context = Yaml::parse(file_get_contents($path));

        Assert::isArray(
            $context,
            "Unable to parse YAML file \"$path\"."
        );

        $modifiedFile = $this->tempDir .'/'. md5($path);

        if (file_exists($modifiedFile)) {

            Assert::readable(
                $modifiedFile,
                'Modified context file of the template is not readable, %s given.'
            );

            $modifiedContext = Yaml::parse(file_get_contents($modifiedFile)) ?: [];

            $context = array_merge($context, $modifiedContext);
        }

        return $context;
    }

    public function getGlobal($name)
    {
        Assert::stringNotEmpty(
            $name,
            'Name of the global template variable must be not empty string, %s given.'
        );

        $globals = $this->getGlobals();

        Assert::keyExists(
            $globals,
            $name,
            'Requested a non-existent global variable %s of template.'
        );

        return $globals[$name];
    }
}