<?php

namespace Moonlight\Service\Parser;

use Glue\Component\HttpFoundation\Request;
use Mikeevstropov\ParameterBag\ParameterBag;
use Moonlight\Utils\ParserTools;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBag as Parameters;
use Webmozart\Assert\Assert;

class PlaylistParser
{
    protected $logger;
    protected $request;
    protected $parameters;

    public function __construct(
        LoggerInterface $logger,
        Request $request,
        Parameters $parameters
    ) {
        $this->logger = $logger;
        $this->request = $request;
        $this->parameters = $parameters;
    }

    /**
     * Gets parsed playlist
     *
     * @param $body
     * @return ParameterBag
     */
    public function getPlaylist($body)
    {
        Assert::stringNotEmpty(
            $body,
            'Body as a source of playlist must be not empty string, %s given.'
        );

        Assert::classExists(
            'DOMDocument',
            'Class DOMDocument not found.'
        );

        libxml_use_internal_errors(true);

        $document = new \DOMDocument();
        $document->loadHTML($body);
        $document->normalizeDocument();

        $query = $this->request->request->all();

        if (isset($query['season'])) unset($query['season']);
        if (isset($query['episode'])) unset($query['episode']);

        $queryString = ParserTools::genPlayerQueryString($query);

        Assert::string(
            $queryString,
            'Query string of playlist url\'s must be a string, %s given.'
        );

        $seasons = [];
        $seasonsEl = $document->getElementById('season');
        $selectedSeason = $this->request->request->get('season');

        if ($seasonsEl) {

            $seasonsEl = $seasonsEl->getElementsByTagName('option');

            /** @var \DOMElement $el */
            foreach ($seasonsEl as $el) {

                $urlQuery = [
                    'season'  => $el->getAttribute('value'),
                    'episode' => 1
                ];

                $urlQuery = http_build_query($urlQuery)
                    . ($queryString ? '&'. $queryString : '');

                $seasons[] = [
                    'selected' => $el->getAttribute('selected') ? true : false,
                    'name'     => $el->getAttribute('value'),
                    'url'      => $this->request->getServer('HTTP_PATH')
                        .'/'. $this->request->getRequest('type')
                        .'/'. $this->request->getRequest('token')
                        .'/iframe?'. $urlQuery
                ];

                if ($el->getAttribute('selected'))
                    $selectedSeason = $el->getAttribute('value');
            }
        }

        $episodes = [];
        $episodesEl = $document->getElementById('episode');
        $selectedEpisode = null;

        if ($episodesEl) {

            $episodesEl = $episodesEl->getElementsByTagName('option');

            foreach ($episodesEl as $el) {

                $urlQuery = [
                    'season'  => $selectedSeason !== null ? $selectedSeason : null,
                    'episode' => $el->getAttribute('value')
                ];

                $urlQuery = http_build_query($urlQuery)
                    . ($queryString ? '&'. $queryString : '');

                $episodes[] = [
                    'name'     => $el->getAttribute('value'),
                    'selected' => $el->getAttribute('selected') ? true : false,
                    'url'      => $this->request->getServer('HTTP_PATH')
                        .'/'. $this->request->getRequest('type')
                        .'/'. $this->request->getRequest('token')
                        .'/iframe?'. $urlQuery
                ];

                if ($el->getAttribute('selected'))
                    $selectedEpisode = $el->getAttribute('value');
            }
        }

        $releases = [];
        $releasesEl = $document->getElementById('translator');
        $selectedRelease = null;

        if ($releasesEl) {

            $releasesEl = $releasesEl->getElementsByTagName('option');

            foreach ($releasesEl as $el) {

                $releases[] = [
                    'name'     => $el->textContent,
                    'selected' => $el->getAttribute('selected') ? true : false,
                    'url'      => $this->request->getServer('HTTP_PATH')
                        .'/'. $this->request->getRequest('type')
                        .'/'. $el->getAttribute('value')
                        .'/iframe'. ($queryString ? '?'. $queryString : '')
                ];

                if ($el->getAttribute('selected'))
                    $selectedRelease = $el->getAttribute('value');
            }
        }

        $this->logger->debug(sprintf(
            'Playlist %d/%d episode of %d/%d season with %d releases.',
            $selectedEpisode,
            count($episodes),
            $selectedSeason,
            count($seasons),
            count($releases)
        ));

        return new ParameterBag([
            'seasons'  => $seasons,
            'episodes' => $episodes,
            'releases' => $releases
        ]);
    }
}