<?php

namespace Glue\Component\HttpKernel;

use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Glue\Component\Controller\ArgumentValueResolver;
use Glue\Component\Controller\ControllerResolver;
use Glue\Component\DependencyInjection\ContainerBuilder;
use Glue\Component\HttpFoundation\Request;
use Glue\Component\HttpFoundation\Response;
use Psr\Log\LoggerInterface;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\DependencyInjection\Loader\YamlFileLoader as ContainerYamlFileLoader;
use Symfony\Component\DependencyInjection\Container;
use Symfony\Component\EventDispatcher\EventDispatcher;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\HttpKernel\Controller\ArgumentResolver;
use Symfony\Component\HttpKernel\HttpKernel;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\Routing\Loader\ClosureLoader;
use Symfony\Component\Routing\RouteCollectionBuilder;
use Symfony\Component\Routing\Loader\YamlFileLoader as RoutesYamlFileLoader;
use Symfony\Component\Routing\Router;
use Webmozart\Assert\Assert;

class Kernel extends KernelAbstract
{
    /**
     * @var string
     */
    protected $rootDir;

    /**
     * @var Request
     */
    protected $request;

    /**
     * @var Router
     */
    protected $router;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Container
     */
    protected $container;

    public function __construct($rootDir)
    {
        Assert::stringNotEmpty(
            $rootDir,
            'Root directory must be not empty string, %s given.'
        );

        Assert::directory(
            $rootDir,
            'Root directory is not directory, %s given.'
        );

        $this->rootDir = realpath($rootDir);
    }

    public function getRootDir()
    {
        return $this->rootDir;
    }

    public function getCacheDir()
    {
        return $this->getRootDir() .'/var/cache';
    }

    public function getLogsDir()
    {
        return $this->getRootDir() .'/var/logs';
    }

    public function getTempDir()
    {
        return $this->getRootDir() .'/var/temp';
    }

    public function getConfigDir()
    {
        return $this->getRootDir() .'/app/config';
    }

    public function configureRequest(Request $request)
    {
        $request->server->set(
            'HTTP_PATH',
            preg_replace('/^(?:.*\/\/[^\/\/]*\/?)?\/?(.*?)(\/?serial.*|\/?video.*|\/?[^\/]*)$/i', '/$1', $_SERVER['REQUEST_URI'])
        );

        $request->server->set(
            'PROTOCOL',
            (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
                || $_SERVER['SERVER_PORT'] == 443 ? 'https' : 'http'
        );
    }

    public function configureRoutes(RouteCollectionBuilder $builder)
    {
        $builder->import(
            $this->getConfigDir() .'/routes.yml',
            $this->request->server->get('HTTP_PATH')
        );
    }

    public function configureRouter(Router $router)
    {
        if (!$this->request->attributes->has('_controller')) {

            $attributes = $router->matchRequest($this->request);
            $this->request->attributes->add($attributes);
        }

        $controllerName = $this->request->attributes->get('_controller');

        Assert::notNull(
            $controllerName,
            'Unable to resolve controller and action name because the "_controller" parameter is missing.'
        );

        $names = ControllerResolver::getNames($controllerName);

        $this->request->attributes->set('_controller_name', $names['controller']);
        $this->request->attributes->set('_action_name',     $names['action']);
    }

    public function configureContainer(ContainerBuilder $container, LoaderInterface $loader)
    {
        $container->setParameter('parameters', $container->getParameterBag());

        $container->setLink('kernel',  $this);
        $container->setLink('request', $this->request);
        $container->setLink('router',  $this->router);
        $container->setLink('logger',  $this->logger);

        $loader->load($this->getConfigDir() .'/parameters.yml');
        $loader->load($this->getConfigDir() .'/services.yml');

        $requestAttributes = $this->request->attributes->all();

        foreach ($requestAttributes as $key => $val) {
            $container->setParameter('request.'. $key, $val);
        }

        $requestServer = $this->request->server->all();

        foreach ($requestServer as $key => $val) {
            $container->setParameter('server.'. strtolower($key), $val);
        }

        $container->setParameter('kernel.root_dir',     $this->getRootDir());
        $container->setParameter('kernel.cache_dir',    $this->getCacheDir());
        $container->setParameter('kernel.logs_dir',     $this->getLogsDir());
        $container->setParameter('kernel.temp_dir',     $this->getTempDir());
        $container->setParameter('kernel.config_dir',   $this->getConfigDir());
    }

    public function configureLogger(LoggerInterface $logger, $requestType)
    {
        if ($requestType === HttpKernelInterface::MASTER_REQUEST) {

            $route    = $this->request->attributes->get('_route');
            $filename = $route .'.route.log';

        } elseif($requestType === HttpKernelInterface::SUB_REQUEST) {

            $route    = 'undefined';
            $filename = 'forward.log';

        } else {

            throw new \LogicException(sprintf(
                'Unable to configure logger with invalid request type, %s given.',
                $requestType
            ));
        }

        $controller = $this->container->getParameter('request._controller');
        $logLevel   = $this->container->hasParameter('log_level')
            ? $this->container->getParameter('log_level')
            : 'DEBUG';

        $streamHandler = new StreamHandler(
            $this->getLogsDir() .'/'. $filename,
            $logLevel
        );

        /** @var \Monolog\Logger $logger */
        $logger->pushHandler($streamHandler);

        $logger->info(sprintf(
            'Start logging by "%s" route of %s controller.',
            $route,
            $controller
        ));
    }

    public function createResponse(ControllerResolver $resolver)
    {
        $response = new Response();

        $response->setLogger($this->logger);

        $dispatcher = new EventDispatcher();

        $argumentResolver = new ArgumentResolver(null, [new ArgumentValueResolver([
            'request'  => $this->request,
            'response' => $response,
            'router'   => $this->router,
            'logger'   => $this->logger,
        ])]);

        $httpKernel = new HttpKernel($dispatcher, $resolver, null, $argumentResolver);

        return $httpKernel->handle($this->request);
    }

    public function onError(\Exception $e)
    {
        $statusCode = $e->getCode() > 0 ? $e->getCode() : 500;
        $statusText = Response::$statusTexts[$statusCode];
        $errMessage = $e->getMessage() ?: 'Unknown error.';

        if ($this->logger)
            $this->logger->critical(sprintf(
                '%d %s',
                $statusCode,
                $errMessage
            ));

        $response = new Response();
        $response->headers->set('content-type', 'text/plain');
        $response->setStatusCode($statusCode);
        $response->setContent(sprintf(
            "%d %s\n\n%s",
            $statusCode,
            $statusText,
            $errMessage
        ));

        $response->send();
    }

    public function handle(Request $request, $type = HttpKernelInterface::MASTER_REQUEST)
    {
        try {

            if ($type === HttpKernelInterface::SUB_REQUEST) {

                $this->logger->debug(sprintf(
                    'Forwarding to the controller %s',
                    $request->attributes->get('_controller')
                ));
            }


            $session = new Session();

            $request->setSession($session);


            $this->configureRequest($request);

            $this->request = $request;


            $routesLoader = new RoutesYamlFileLoader(new FileLocator());

            $routesBuilder = new RouteCollectionBuilder($routesLoader);

            $this->configureRoutes($routesBuilder);

            $routes = $routesBuilder->build();

            $this->router = new Router(new ClosureLoader(), function () use ($routes) { return $routes; });

            $this->configureRouter($this->router);


            $this->container = new ContainerBuilder();

            $containerLoader = new ContainerYamlFileLoader($this->container, new FileLocator());

            $this->configureContainer($this->container, $containerLoader);


            $this->logger = new Logger('logger');

            $this->configureLogger($this->logger, $type);


            $controllerResolver = new ControllerResolver($this->container, $this->logger);

            $response = $this->createResponse($controllerResolver);

        } catch (\Exception $e) {

            $this->onError($e);

            die();

        }

        return $response;
    }
}